@@ -23,6 +23,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "main/main_app_config.h"
 #include "main/main_session.h"
 #include "payments/payments_form.h"
+#include "ui/chat/chat_style.h" // ColorCollectible
 #include "ui/text/format_values.h"
 
 namespace Api {
@@ -35,7 +36,7 @@ namespace {
 		.giveawayId = data.vgiveaway_msg_id().value_or_empty(),
 		.date = data.vdate().v,
 		.used = data.vused_date().value_or_empty(),
-		.months = data.vmonths().v,
+		.days = data.vdays().v,
 		.giveaway = data.is_via_giveaway(),
 	};
 }
@@ -857,6 +858,8 @@ std::optional<Data::StarGift> FromTL(
 			.releasedBy = releasedBy,
 			.resellTitle = qs(data.vtitle().value_or_empty()),
 			.resellCount = int(data.vavailability_resale().value_or_empty()),
+			.auctionSlug = qs(data.vauction_slug().value_or_empty()),
+			.auctionGiftsPerRound = data.vgifts_per_round().value_or_empty(),
 			.limitedLeft = remaining.value_or_empty(),
 			.limitedCount = total.value_or_empty(),
 			.perUserTotal = data.vper_user_total().value_or_empty(),
@@ -865,6 +868,7 @@ std::optional<Data::StarGift> FromTL(
 			.lastSaleDate = data.vlast_sale_date().value_or_empty(),
 			.lockedUntilDate = data.vlocked_until_date().value_or_empty(),
 			.requirePremium = data.is_require_premium(),
+			.peerColorAvailable = data.is_peer_color_available(),
 			.upgradable = data.vupgrade_stars().has_value(),
 			.birthday = data.is_birthday(),
 			.soldOut = data.is_sold_out(),
@@ -900,6 +904,12 @@ std::optional<Data::StarGift> FromTL(
 		const auto themeUser = themeUserId
 			? session->data().peer(themeUserId).get()
 			: nullptr;
+		const auto colorCollectible = (data.vpeer_color()
+			&& data.vpeer_color()->type() == mtpc_peerColorCollectible)
+			? std::make_shared<Ui::ColorCollectible>(
+				Data::ParseColorCollectible(
+					data.vpeer_color()->c_peerColorCollectible()))
+			: nullptr;
 		auto result = Data::StarGift{
 			.id = data.vid().v,
 			.unique = std::make_shared<Data::UniqueGift>(Data::UniqueGift{
@@ -907,11 +917,15 @@ std::optional<Data::StarGift> FromTL(
 				.initialGiftId = data.vgift_id().v,
 				.slug = qs(data.vslug()),
 				.title = qs(data.vtitle()),
+				.giftAddress = qs(data.vgift_address().value_or_empty()),
 				.ownerAddress = qs(data.vowner_address().value_or_empty()),
 				.ownerName = qs(data.vowner_name().value_or_empty()),
 				.ownerId = (data.vowner_id()
 					? peerFromMTP(*data.vowner_id())
 					: PeerId()),
+				.hostId = (data.vhost_id()
+					? peerFromMTP(*data.vhost_id())
+					: PeerId()),
 				.releasedBy = releasedBy,
 				.themeUser = themeUser,
 				.nanoTonForResale = FindTonForResale(data.vresell_amount()),
@@ -930,6 +944,7 @@ std::optional<Data::StarGift> FromTL(
 								data.vvalue_amount().value_or_empty()),
 						})
 					: nullptr),
+				.peerColor = colorCollectible,
 			}),
 			.document = model->document,
 			.releasedBy = releasedBy,
@@ -986,6 +1001,8 @@ std::optional<Data::SavedStarGift> FromTL(
 		.starsConverted = int64(data.vconvert_stars().value_or_empty()),
 		.starsUpgradedBySender = int64(
 			data.vupgrade_stars().value_or_empty()),
+		.starsForDetailsRemove = int64(
+			data.vdrop_original_details_stars().value_or_empty()),
 		.giftPrepayUpgradeHash = qs(
 			data.vprepaid_upgrade_hash().value_or_empty()),
 		.fromId = (data.vfrom_id()
