@@ -586,7 +586,7 @@ void ShareBox::showMenu(not_null<Ui::RpWidget*> parent) {
 		uiShow()->showBox(
 			HistoryView::PrepareScheduleBox(
 				this,
-				nullptr, // ChatHelpers::Show for effect attachment.
+				_descriptor.session,
 				sendMenuDetails(),
 				[=](Api::SendOptions options) { submit(options); },
 				action.options,
@@ -1707,6 +1707,9 @@ ShareBox::SubmitCallback ShareBox::DefaultForwardCallback(
 		const auto commonSendFlags = Flag(0)
 			| Flag::f_with_my_score
 			| (options.scheduled ? Flag::f_schedule_date : Flag(0))
+			| ((options.scheduled && options.scheduleRepeatPeriod)
+				? Flag::f_schedule_repeat_period
+				: Flag(0))
 			| ((forwardOptions != Data::ForwardOptions::PreserveInfo)
 				? Flag::f_drop_author
 				: Flag(0))
@@ -1786,6 +1789,7 @@ ShareBox::SubmitCallback ShareBox::DefaultForwardCallback(
 							? MTP_inputReplyToMonoForum(sublistPeer->input)
 							: MTPInputReplyTo()),
 						MTP_int(options.scheduled),
+						MTP_int(options.scheduleRepeatPeriod),
 						MTP_inputPeerEmpty(), // send_as
 						Data::ShortcutIdToMTP(session, options.shortcutId),
 						MTP_int(videoTimestamp.value_or(0)),
