@@ -104,11 +104,13 @@ struct CreditsHistoryEntry final {
 	bool converted : 1 = false;
 	bool anonymous : 1 = false;
 	bool stargift : 1 = false;
+	bool auction : 1 = false;
 	bool postsSearch : 1 = false;
 	bool giftTransferred : 1 = false;
 	bool giftRefunded : 1 = false;
 	bool giftUpgraded : 1 = false;
 	bool giftUpgradeSeparate : 1 = false;
+	bool giftUpgradeGifted : 1 = false;
 	bool giftResale : 1 = false;
 	bool giftResaleForceTon : 1 = false;
 	bool giftPinned : 1 = false;
