@@ -7,18 +7,18 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "data/data_shared_media.h"
 
-#include <rpl/combine.h>
-#include "main/main_session.h"
 #include "apiwrap.h"
-#include "storage/storage_facade.h"
-#include "history/history.h"
-#include "history/history_item.h"
+#include "core/crash_reports.h"
 #include "data/components/scheduled_messages.h"
 #include "data/data_document.h"
 #include "data/data_media_types.h"
 #include "data/data_photo.h"
+#include "data/data_saved_music.h"
 #include "data/data_session.h"
-#include "core/crash_reports.h"
+#include "history/history.h"
+#include "history/history_item.h"
+#include "main/main_session.h"
+#include "storage/storage_facade.h"
 
 namespace {
 
@@ -233,6 +233,39 @@ rpl::producer<SparseIdsMergedSlice> SharedScheduledMediaViewer(
 	});
 }
 
+rpl::producer<SparseIdsMergedSlice> SavedMusicMediaViewer(
+		not_null<Main::Session*> session,
+		SharedMediaMergedKey key,
+		int limitBefore,
+		int limitAfter) {
+	Expects((key.mergedKey.universalId != 0)
+		|| (limitBefore == 0 && limitAfter == 0));
+
+	const auto peerId = key.mergedKey.peerId;
+	const auto item = key.mergedKey.universalId
+		? session->data().message(peerId, key.mergedKey.universalId)
+		: nullptr;
+
+	return Data::SavedMusicList(
+		session->data().peer(peerId),
+		item,
+		std::max(limitBefore, limitAfter)
+	) | rpl::map([=](const Data::SavedMusicSlice &slice) {
+		auto list = std::vector<MsgId>();
+		list.reserve(slice.size());
+		for (auto i = 0, count = int(slice.size()); i != count; ++i) {
+			list.push_back(slice[i]->id);
+		}
+		return SparseIdsMergedSlice(
+			key.mergedKey,
+			SparseUnsortedIdsSlice(
+				std::move(list),
+				slice.fullCount(),
+				slice.skippedBefore(),
+				slice.skippedAfter()));
+	});
+}
+
 rpl::producer<SparseIdsMergedSlice> SharedMediaMergedViewer(
 		not_null<Main::Session*> session,
 		SharedMediaMergedKey key,
@@ -490,6 +523,19 @@ rpl::producer<SharedMediaWithLastSlice> SharedMediaWithLastViewer(
 					std::move(update),
 					std::nullopt));
 			});
+		} else if (key.topicRootId == SharedMediaWithLastSlice::kSavedMusicTopicId) {
+			return SavedMusicMediaViewer(
+				session,
+				std::move(viewerKey),
+				limitBefore,
+				limitAfter
+			) | rpl::start_with_next([=](SparseIdsMergedSlice &&update) {
+				consumer.put_next(SharedMediaWithLastSlice(
+					session,
+					key,
+					std::move(update),
+					std::nullopt));
+			});
 		}
 		return rpl::combine(
 			SharedMediaMergedViewer(
