@@ -14,6 +14,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_changes.h"
 #include "data/data_channel.h"
 #include "data/data_document.h"
+#include "data/data_group_call.h"
 #include "data/data_folder.h"
 #include "data/data_photo.h"
 #include "data/data_user.h"
@@ -49,12 +50,13 @@ constexpr auto kPollingViewsPerPage = Story::kRecentViewersMax;
 using UpdateFlag = StoryUpdate::Flag;
 
 [[nodiscard]] std::optional<StoryMedia> ParseMedia(
-		not_null<Session*> owner,
-		const MTPMessageMedia &media) {
+		not_null<PeerData*> peer,
+		const MTPMessageMedia &media,
+		const std::shared_ptr<GroupCall> &existingCall) {
 	return media.match([&](const MTPDmessageMediaPhoto &data)
 		-> std::optional<StoryMedia> {
 		if (const auto photo = data.vphoto()) {
-			const auto result = owner->processPhoto(*photo);
+			const auto result = peer->owner().processPhoto(*photo);
 			if (!result->isNull()) {
 				return StoryMedia{ result };
 			}
@@ -63,7 +65,7 @@ using UpdateFlag = StoryUpdate::Flag;
 	}, [&](const MTPDmessageMediaDocument &data)
 		-> std::optional<StoryMedia> {
 		if (const auto document = data.vdocument()) {
-			const auto result = owner->processDocument(
+			const auto result = peer->owner().processDocument(
 				*document,
 				data.valt_documents());
 			if (!result->isNull()
@@ -73,6 +75,25 @@ using UpdateFlag = StoryUpdate::Flag;
 			}
 		}
 		return {};
+	}, [&](const MTPDmessageMediaVideoStream &data) {
+		const auto rtmp = data.is_rtmp_stream();
+		return std::make_optional(data.vcall().match([&](
+				const MTPDinputGroupCall &data) {
+			auto call = (existingCall
+				&& existingCall->peer() == peer
+				&& existingCall->id() == data.vid().v)
+				? existingCall
+				: std::make_shared<Data::GroupCall>(
+					peer,
+					data.vid().v,
+					data.vaccess_hash().v,
+					TimeId(),
+					rtmp,
+					GroupCallOrigin::VideoStream);
+			return StoryMedia{ std::move(call) };
+		}, [](const auto &) {
+			return StoryMedia();
+		}));
 	}, [&](const MTPDmessageMediaUnsupported &data) {
 		return std::make_optional(StoryMedia{ v::null });
 	}, [](const auto &) { return std::optional<StoryMedia>(); });
@@ -145,6 +166,7 @@ StoriesSourceInfo StoriesSource::info() const {
 		.count = uint32(std::min(int(ids.size()), kMaxSegmentsCount)),
 		.unreadCount = uint32(std::min(unreadCount(), kMaxSegmentsCount)),
 		.premium = (peer->isUser() && peer->asUser()->isPremium()) ? 1U : 0,
+		.hasVideoStream = hasVideoStream ? 1U : 0,
 	};
 }
 
@@ -422,6 +444,13 @@ void Stories::parseAndApply(
 	for (const auto &story : list) {
 		if (const auto id = parseAndApply(result.peer, story, now)) {
 			result.ids.emplace(id);
+
+			if (story.type() == mtpc_storyItem) {
+				const auto &media = story.c_storyItem().vmedia();
+				if (media.type() == mtpc_messageMediaVideoStream) {
+					result.hasVideoStream = true;
+				}
+			}
 		}
 	}
 	if (result.ids.empty()) {
@@ -494,7 +523,14 @@ Story *Stories::parseAndApply(
 		not_null<PeerData*> peer,
 		const MTPDstoryItem &data,
 		TimeId now) {
-	const auto media = ParseMedia(_owner, data.vmedia());
+	const auto id = data.vid().v;
+	const auto fullId = FullStoryId{ peer->id, id };
+	auto &stories = _stories[peer->id];
+	const auto i = stories.find(id);
+	const auto &existingCall = (i != end(stories))
+		? i->second->call()
+		: nullptr;
+	const auto media = ParseMedia(peer, data.vmedia(), existingCall);
 	if (!media) {
 		return nullptr;
 	}
@@ -503,7 +539,6 @@ Story *Stories::parseAndApply(
 	if (expired && !data.is_pinned() && !hasArchive(peer)) {
 		return nullptr;
 	}
-	const auto id = data.vid().v;
 	auto albumInfo = (Albums*)nullptr;
 	auto list = std::optional<base::flat_set<int>>();
 	if (const auto albums = data.valbums()) {
@@ -517,9 +552,6 @@ Story *Stories::parseAndApply(
 			}
 		}
 	}
-	const auto fullId = FullStoryId{ peer->id, id };
-	auto &stories = _stories[peer->id];
-	const auto i = stories.find(id);
 	if (i != end(stories)) {
 		const auto result = i->second.get();
 		const auto mediaChanged = (result->media() != *media);
@@ -1974,6 +2006,58 @@ void Stories::albumDelete(not_null<PeerData*> peer, int id) {
 	}
 }
 
+void Stories::albumReorderStories(
+		not_null<PeerData*> peer,
+		int albumId,
+		int oldPosition,
+		int newPosition,
+		Fn<void()> done,
+		Fn<void()> fail) {
+	const auto ids = albumIds(peer->id, albumId);
+	const auto list = RespectingPinned(ids);
+
+	if (oldPosition < 0 || newPosition < 0
+		|| oldPosition >= list.size() || newPosition >= list.size()) {
+		fail();
+		return;
+	}
+
+	if (_reorderStoriesRequestId) {
+		_owner->session().api().request(
+			base::take(_reorderStoriesRequestId)).cancel();
+	}
+
+	auto reorderedList = list;
+	base::reorder(reorderedList, oldPosition, newPosition);
+
+	auto order = QVector<MTPint>();
+	order.reserve(reorderedList.size());
+	for (const auto id : reorderedList) {
+		order.push_back(MTP_int(id));
+	}
+
+	_reorderStoriesRequestId = _owner->session().api().request(
+		MTPstories_UpdateAlbum(
+			MTP_flags(MTPstories_UpdateAlbum::Flag::f_order),
+			peer->input,
+			MTP_int(albumId),
+			MTPstring(),
+			MTPVector<MTPint>(),
+			MTPVector<MTPint>(),
+			MTP_vector<MTPint>(order)
+	)).done([=](const MTPStoryAlbum &result) {
+		_reorderStoriesRequestId = 0;
+		if (const auto set = albumIdsSet(peer->id, albumId)) {
+			set->ids.list = reorderedList;
+			_albumIdsChanged.fire({ peer->id, albumId });
+		}
+		done();
+	}).fail([=] {
+		_reorderStoriesRequestId = 0;
+		fail();
+	}).send();
+}
+
 void Stories::notifyAlbumUpdate(StoryAlbumUpdate &&update) {
 	const auto peerId = update.peer->id;
 	const auto i = _albums.find(peerId);
@@ -2254,18 +2338,25 @@ void Stories::setPreloadingInViewer(std::vector<FullStoryId> ids) {
 
 std::optional<Stories::PeerSourceState> Stories::peerSourceState(
 		not_null<PeerData*> peer,
-		StoryId storyMaxId) {
+		const MTPRecentStory &recent) {
+	const auto &data = recent.data();
+	const auto maxId = data.vmax_id().value_or_empty();
+	const auto live = data.is_live();
 	const auto i = _readTill.find(peer->id);
 	if (_readTillReceived || (i != end(_readTill))) {
 		return PeerSourceState{
-			.maxId = storyMaxId,
+			.maxId = maxId,
 			.readTill = std::min(
-				storyMaxId,
+				maxId,
 				(i != end(_readTill)) ? i->second : 0),
+			.hasVideoStream = live,
 		};
 	}
 	requestReadTills();
-	_pendingPeerStateMaxId[peer] = storyMaxId;
+	_pendingPeerRecentState[peer] = RecentState{
+		.maxId = maxId,
+		.hasVideoStream = live,
+	};
 	return std::nullopt;
 }
 
@@ -2278,8 +2369,8 @@ void Stories::requestReadTills() {
 	)).done([=](const MTPUpdates &result) {
 		_readTillReceived = true;
 		api->applyUpdates(result);
-		for (auto &[peer, maxId] : base::take(_pendingPeerStateMaxId)) {
-			updatePeerStoriesState(peer);
+		for (auto &[peer, recent] : base::take(_pendingPeerRecentState)) {
+			updatePeerStoriesState(peer, recent);
 		}
 		for (const auto &storyId : base::take(_pendingReadTillItems)) {
 			_owner->refreshStoryItemViews(storyId);
@@ -2403,20 +2494,32 @@ void Stories::sendPollingViewsRequests() {
 	_pollingViewsTimer.callOnce(kPollViewsInterval);
 }
 
-void Stories::updatePeerStoriesState(not_null<PeerData*> peer) {
+void Stories::updatePeerStoriesState(
+		not_null<PeerData*> peer,
+		std::optional<RecentState> cachedRecentState) {
 	const auto till = _readTill.find(peer->id);
 	const auto readTill = (till != end(_readTill)) ? till->second : 0;
-	const auto pendingMaxId = [&] {
-		const auto j = _pendingPeerStateMaxId.find(peer);
-		return (j != end(_pendingPeerStateMaxId)) ? j->second : 0;
+	const auto pendingRecentState = [&] {
+		if (cachedRecentState) {
+			return *cachedRecentState;
+		}
+		const auto j = _pendingPeerRecentState.find(peer);
+		return (j != end(_pendingPeerRecentState))
+			? j->second
+			: RecentState();
 	};
 	const auto i = _all.find(peer->id);
-	const auto max = (i != end(_all))
-		? (i->second.ids.empty() ? 0 : i->second.ids.back().id)
-		: pendingMaxId();
-	peer->setStoriesState(!max
+	const auto recent = (i != end(_all))
+		? RecentState{
+			(i->second.ids.empty() ? 0 : i->second.ids.back().id),
+			(!i->second.ids.empty() && i->second.ids.back().videoStream),
+		}
+		: pendingRecentState();
+	peer->setStoriesState(recent.hasVideoStream
+		? PeerData::StoriesState::HasVideoStream
+		: !recent.maxId
 		? PeerData::StoriesState::None
-		: (max <= readTill)
+		: (recent.maxId <= readTill)
 		? PeerData::StoriesState::HasRead
 		: PeerData::StoriesState::HasUnread);
 }
