@@ -89,6 +89,10 @@ struct StoriesInfo {
 	int count = 0;
 };
 
+struct ProfileMusicInfo {
+	int count = 0;
+};
+
 struct FileLocation {
 	int dcId = 0;
 	MTPInputFileLocation data;
@@ -612,7 +616,7 @@ struct ActionWebViewDataSent {
 
 struct ActionGiftPremium {
 	Utf8String cost;
-	int months = 0;
+	int days = 0;
 };
 
 struct ActionTopicCreate {
@@ -637,7 +641,7 @@ struct ActionSetChatWallPaper {
 struct ActionGiftCode {
 	QByteArray code;
 	PeerId boostPeerId = 0;
-	int months = 0;
+	int days = 0;
 	bool viaGiveaway = false;
 	bool unclaimed = false;
 };
@@ -929,6 +933,11 @@ StoriesSlice ParseStoriesSlice(
 	const MTPVector<MTPStoryItem> &data,
 	int baseIndex);
 
+struct ProfileMusicSlice {
+	std::vector<Message> list;
+	int skipped = 0;
+};
+
 Message ParseMessage(
 	ParseMediaContext &context,
 	const MTPMessage &data,
