@@ -7,12 +7,17 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #pragma once
 
+#include "api/api_common.h"
+
 namespace Api {
 enum class SendProgressType;
-struct SendOptions;
 struct SendAction;
 } // namespace Api
 
+namespace Data {
+class GroupCall;
+} // namespace Data
+
 class History;
 
 namespace HistoryView::Controls {
@@ -41,6 +46,7 @@ enum class WriteRestrictionType {
 	Rights,
 	PremiumRequired,
 	Frozen,
+	Hidden,
 };
 
 struct WriteRestriction {
@@ -65,6 +71,7 @@ struct WriteRestriction {
 
 struct SetHistoryArgs {
 	required<History*> history;
+	std::shared_ptr<Data::GroupCall> videoStream;
 	MsgId topicRootId = 0;
 	PeerId monoforumPeerId = 0;
 	Fn<bool()> showSlowmodeError;
@@ -72,6 +79,7 @@ struct SetHistoryArgs {
 	rpl::producer<int> slowmodeSecondsLeft;
 	rpl::producer<bool> sendDisabledBySlowmode;
 	rpl::producer<bool> liked;
+	rpl::producer<int> minStarsCount;
 	rpl::producer<WriteRestriction> writeRestriction;
 };
 
@@ -84,4 +92,11 @@ struct ReplyNextRequest {
 	const Direction direction;
 };
 
+enum class ToggleCommentsState {
+	Empty,
+	Shown,
+	Hidden,
+	WithNew,
+};
+
 } // namespace HistoryView::Controls
