@@ -7,6 +7,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "history/view/media/history_view_unique_gift.h"
 
+#include "base/unixtime.h"
 #include "boxes/star_gift_box.h"
 #include "chat_helpers/stickers_lottie.h"
 #include "core/click_handler_types.h"
@@ -15,6 +16,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_media_types.h"
 #include "data/data_session.h"
 #include "data/data_star_gift.h"
+#include "data/data_web_page.h"
 #include "history/view/media/history_view_media_generic.h"
 #include "history/view/media/history_view_premium_gift.h"
 #include "history/view/history_view_cursor_state.h"
@@ -26,6 +28,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "main/main_session.h"
 #include "settings/settings_credits_graphics.h"
 #include "ui/chat/chat_style.h"
+#include "ui/effects/ministar_particles.h"
 #include "ui/effects/premium_stars_colored.h"
 #include "ui/effects/ripple_animation.h"
 #include "ui/layers/generic_box.h"
@@ -33,6 +36,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "ui/painter.h"
 #include "ui/power_saving.h"
 #include "ui/rect.h"
+#include "ui/top_background_gradient.h"
 #include "window/window_session_controller.h"
 #include "styles/style_chat.h"
 #include "styles/style_credits.h"
@@ -476,9 +480,9 @@ auto UniqueGiftBg(
 		const auto top = (webpreview ? 2 : 1) * (-shift);
 		const auto outer = QRect(-shift, top, doubled, doubled);
 		p.setClipRect(inner);
-		Ui::PaintPoints(
+		Ui::PaintBgPoints(
 			p,
-			Ui::PatternPoints(),
+			Ui::PatternBgPoints(),
 			state->cache,
 			state->pattern.get(),
 			*gift,
@@ -535,6 +539,187 @@ auto GenerateUniqueGiftPreview(
 	};
 }
 
+auto GenerateAuctionPreview(
+	not_null<Element*> parent,
+	Element *replacing,
+	std::shared_ptr<Data::StarGift> gift,
+	Data::UniqueGiftBackdrop backdrop,
+	TimeId endDate)
+-> Fn<void(
+		not_null<MediaGeneric*>,
+		Fn<void(std::unique_ptr<MediaGenericPart>)>)> {
+	return [=](
+			not_null<MediaGeneric*> media,
+			Fn<void(std::unique_ptr<MediaGenericPart>)> push) {
+		const auto sticker = [=] {
+			using Tag = ChatHelpers::StickerLottieSize;
+			return StickerInBubblePart::Data{
+				.sticker = gift->document,
+				.size = st::chatIntroStickerSize,
+				.cacheTag = Tag::ChatIntroHelloSticker,
+			};
+		};
+		push(std::make_unique<StickerInBubblePart>(
+			parent,
+			replacing,
+			sticker,
+			st::webPageAuctionPreviewPadding));
+		const auto name = gift->unique
+			? Data::UniqueGiftName(*gift->unique)
+			: gift->resellTitle;
+		if (!name.isEmpty()) {
+			push(std::make_unique<TextPartColored>(
+				Ui::Text::Bold(name),
+				QMargins(0, 0, 0, st::defaultVerticalListSkip),
+				[c = backdrop.textColor](const auto&) { return c; },
+				st::chatUniqueTitle));
+		}
+		if (const auto all = gift->limitedCount) {
+			push(std::make_unique<TextPartColored>(
+				tr::lng_boosts_list_tab_gifts(
+					tr::now,
+					lt_count,
+					all,
+					Ui::Text::WithEntities),
+				QMargins(0, 0, 0, st::webPageAuctionPreviewPadding.top()),
+				[c = backdrop.textColor](const auto&) { return c; },
+				st::chatUniqueTextStyle));
+		}
+	};
+}
+
+auto AuctionBg(
+	not_null<Element*> view,
+	Data::UniqueGiftBackdrop backdrop,
+	std::shared_ptr<Data::StarGift> gift,
+	TimeId endDate)
+-> Fn<void(
+		Painter&,
+		const Ui::ChatPaintContext&,
+		not_null<const MediaGeneric*>)> {
+	struct State {
+		std::unique_ptr<Ui::Text::CustomEmoji> pattern;
+		base::flat_map<float64, QImage> cache;
+		std::optional<Ui::StarParticles> particles;
+		std::unique_ptr<base::Timer> timer;
+		crl::time pausedAt = 0;
+		crl::time pauseOffset = 0;
+	};
+	const auto state = std::make_shared<State>();
+	if (gift->unique && gift->unique->pattern.document) {
+		state->pattern = view->history()->owner().customEmojiManager().create(
+			gift->unique->pattern.document,
+			[=] { view->repaint(); },
+			Data::CustomEmojiSizeTag::Large);
+	}
+	state->particles.emplace(
+		Ui::StarParticles::Type::RadialInside,
+		25,
+		st::lineWidth * 8);
+	state->particles->setSpeed(0.05);
+	state->particles->setColor(backdrop.textColor);
+
+	return [=](
+			Painter &p,
+			const Ui::ChatPaintContext &context,
+			not_null<const MediaGeneric*> media) {
+		auto hq = PainterHighQualityEnabler(p);
+		p.setPen(Qt::NoPen);
+		const auto webpreview = (media.get() != view->media());
+		const auto radius = webpreview
+			? st::roundRadiusLarge
+			: st::msgServiceGiftBoxRadius;
+		const auto full = QRect(0, 0, media->width(), media->height());
+		auto gradient = QRadialGradient(full.center(), full.height() / 2);
+		gradient.setStops({
+			{ 0., backdrop.centerColor },
+			{ 1., backdrop.edgeColor },
+		});
+		p.setBrush(gradient);
+		p.drawRoundedRect(full, radius, radius);
+
+		/*if (state->pattern) {
+			const auto width = media->width();
+			const auto shift = width / 12;
+			const auto doubled = width + 2 * shift;
+			const auto top = (webpreview ? 2 : 1) * (-shift);
+			const auto outer = QRect(-shift, top, doubled, doubled);
+			p.setClipRect(full);
+			if (gift->unique) {
+				Ui::PaintBgPoints(
+					p,
+					Ui::PatternBgPoints(),
+					state->cache,
+					state->pattern.get(),
+					*gift->unique,
+					outer);
+			}
+			p.setClipping(false);
+		}*/
+
+		if (state->particles) {
+			p.setClipRect(full);
+			if (context.paused) {
+				if (!state->pausedAt) {
+					state->pausedAt = crl::now();
+				}
+				const auto diff = state->pausedAt - state->pauseOffset;
+				state->particles->paint(p, full, diff);
+			} else {
+				if (state->pausedAt) {
+					state->pauseOffset += crl::now() - state->pausedAt;
+					state->pausedAt = 0;
+				}
+				const auto diff = context.now - state->pauseOffset;
+				state->particles->paint(p, full, diff);
+			}
+			p.setClipping(false);
+		}
+
+		const auto now = base::unixtime::now();
+		const auto left = std::max(endDate - now, 0);
+		if (left > 0) {
+			if (!state->timer) {
+				state->timer = std::make_unique<base::Timer>([=] {
+					view->repaint();
+				});
+			}
+			state->timer->callOnce(1000);
+		} else if (left <= 0 && state->timer) {
+			state->timer = nullptr;
+		}
+		const auto text = left > 0
+			? QString("%1:%2:%3")
+				.arg(left / 3600, 2, 10, QChar('0'))
+				.arg((left % 3600) / 60, 2, 10, QChar('0'))
+				.arg(left % 60, 2, 10, QChar('0'))
+			: tr::lng_auctino_preview_finished(tr::now);
+
+		const auto &font = st::webPageAuctionTimeFont;
+		const auto textWidth = font->width(text);
+		const auto padding = st::webPageAuctionTimerPadding;
+		const auto timerWidth = textWidth + rect::m::sum::h(padding);
+		const auto timerHeight = font->height + rect::m::sum::v(padding);
+		const auto timerRadius = timerHeight / 2.;
+		const auto timerRect = QRectF(
+			padding.top(),
+			padding.top(),
+			timerWidth,
+			timerHeight);
+
+		p.setPen(Qt::NoPen);
+		p.setBrush(st::slideFadeOutBg);
+		p.drawRoundedRect(timerRect, timerRadius, timerRadius);
+
+		p.setPen(backdrop.textColor);
+		p.setFont(font);
+		p.drawText(
+			timerRect.x() + padding.left(),
+			timerRect.y() + padding.top() + font->ascent,
+			text);
+	};
+}
+
 std::unique_ptr<MediaGenericPart> MakeGenericButtonPart(
 		const QString &text,
 		QMargins margins,
