@@ -48,6 +48,7 @@ class AbstractController;
 namespace Media {
 
 struct ListFoundItem;
+struct ListFoundItemWithSection;
 struct ListContext;
 class ListSection;
 class ListProvider;
@@ -70,6 +71,13 @@ public:
 	rpl::producer<SelectedItems> selectedListValue() const;
 	void selectionAction(SelectionAction action);
 
+	struct ReorderDescriptor {
+		Fn<void(int old, int pos, Fn<void()> done, Fn<void()> fail)> save;
+		Fn<bool(HistoryItem*)> filter;
+	};
+
+	void setReorderDescriptor(ReorderDescriptor descriptor);
+
 	QRect getCurrentSongGeometry();
 	rpl::producer<> checkForHide() const {
 		return _checkForHide.events();
@@ -113,6 +121,24 @@ private:
 		Dragging,
 		PrepareSelect,
 		Selecting,
+		PrepareReorder,
+		Reordering,
+	};
+	struct ReorderState {
+		bool enabled = false;
+		int index = -1;
+		int targetIndex = -1;
+		QPoint startPos;
+		QPoint dragPoint;
+		QPoint currentPos;
+		BaseLayout *item = nullptr;
+		const Section *section = nullptr;
+	};
+	struct ShiftAnimation {
+		Ui::Animations::Simple xAnimation;
+		Ui::Animations::Simple yAnimation;
+		int shift = 0;
+		int targetShift = 0;
 	};
 	struct MouseState {
 		HistoryItem *item = nullptr;
@@ -233,7 +259,10 @@ private:
 	[[nodiscard]] auto findSectionAfterBottom(
 		std::vector<Section>::const_iterator from,
 		int bottom) const -> std::vector<Section>::const_iterator;
+	[[nodiscard]] auto findSectionAndItem(QPoint point) const
+		-> std::pair<std::vector<Section>::const_iterator, FoundItem>;
 	[[nodiscard]] FoundItem findItemByPoint(QPoint point) const;
+	[[nodiscard]] ListFoundItemWithSection findItemByPointWithSection(QPoint point) const;
 	[[nodiscard]] std::optional<FoundItem> findItemByItem(
 		const HistoryItem *item);
 	[[nodiscard]] FoundItem findItemDetails(not_null<BaseLayout*> item);
@@ -279,6 +308,19 @@ private:
 
 	void setupStoriesTrackIds();
 
+	void startReorder(const QPoint &globalPos);
+	void updateReorder(const QPoint &globalPos);
+	void finishReorder();
+	void cancelReorder();
+	void updateShiftAnimations();
+	[[nodiscard]] int itemIndexFromPoint(QPoint point) const;
+	[[nodiscard]] QRect itemGeometryByIndex(int index);
+	[[nodiscard]] BaseLayout *itemByIndex(int index);
+	[[nodiscard]] bool canReorder() const;
+	void reorderItemsInSections(int oldIndex, int newIndex);
+	void resetAllItemShifts();
+	void finishShiftAnimations();
+
 	const not_null<AbstractController*> _controller;
 	const std::unique_ptr<ListProvider> _provider;
 
@@ -327,6 +369,13 @@ private:
 
 	base::flat_map<not_null<Main::Session*>, rpl::lifetime> _trackedSessions;
 
+	ReorderState _reorderState;
+	base::flat_map<int, ShiftAnimation> _shiftAnimations;
+	int _activeShiftAnimations = 0;
+	Ui::Animations::Simple _returnAnimation;
+	ReorderDescriptor _reorderDescriptor;
+	bool _inDragArea = false;
+
 };
 
 } // namespace Media
