@@ -7,6 +7,7 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 */
 #include "info/profile/info_profile_badge.h"
 
+#include "data/data_changes.h"
 #include "data/data_emoji_statuses.h"
 #include "data/data_peer.h"
 #include "data/data_session.h"
@@ -90,17 +91,24 @@ void Badge::setContent(Content content) {
 			? (Data::FrameSizeFromTag(sizeTag())
 				/ style::DevicePixelRatio())
 			: 0;
+		const auto &style = st();
 		const auto icon = (_content.badge == BadgeType::Verified)
-			? &_st.verified
+			? &style.verified
 			: id
 			? nullptr
-			: &_st.premium;
+			: &style.premium;
+		const auto iconForeground = (_content.badge == BadgeType::Verified)
+			? &style.verifiedCheck
+			: nullptr;
 		if (id) {
 			_emojiStatus = _session->data().customEmojiManager().create(
 				Data::EmojiStatusCustomId(id),
 				[raw = _view.data()] { raw->update(); },
 				sizeTag());
-			if (_customStatusLoopsLimit > 0) {
+			if (_content.badge == BadgeType::BotVerified) {
+				_emojiStatus = std::make_unique<Ui::Text::FirstFrameEmoji>(
+					std::move(_emojiStatus));
+			} else if (_customStatusLoopsLimit > 0) {
 				_emojiStatus = std::make_unique<Ui::Text::LimitedLoopsEmoji>(
 					std::move(_emojiStatus),
 					_customStatusLoopsLimit);
@@ -113,7 +121,7 @@ void Badge::setContent(Content content) {
 		) | rpl::start_with_next([=, check = _view.data()]{
 			if (_emojiStatus) {
 				auto args = Ui::Text::CustomEmoji::Context{
-					.textColor = _st.premiumFg->c,
+					.textColor = style.premiumFg->c,
 					.now = crl::now(),
 					.paused = ((_animationPaused && _animationPaused())
 						|| On(PowerSaving::kEmojiStatus)),
@@ -125,8 +133,29 @@ void Badge::setContent(Content content) {
 				}
 			}
 			if (icon) {
-				Painter p(check);
-				icon->paint(p, emoji, 0, check->width());
+				auto p = Painter(check);
+				if (_overrideSt && !iconForeground) {
+					icon->paint(
+						p,
+						emoji,
+						0,
+						check->width(),
+						_overrideSt->premiumFg->c);
+				} else {
+					icon->paint(p, emoji, 0, check->width());
+				}
+				if (iconForeground) {
+					if (_overrideSt) {
+						iconForeground->paint(
+							p,
+							emoji,
+							0,
+							check->width(),
+							_overrideSt->premiumFg->c);
+					} else {
+						iconForeground->paint(p, emoji, 0, check->width());
+					}
+				}
 			}
 		}, _view->lifetime());
 	} break;
@@ -179,6 +208,13 @@ void Badge::setPremiumClickCallback(Fn<void()> callback) {
 	}
 }
 
+void Badge::setOverrideStyle(const style::InfoPeerBadge *st) {
+	const auto was = _content;
+	_overrideSt = st;
+	_content = {};
+	setContent(was);
+}
+
 rpl::producer<> Badge::updated() const {
 	return _updated.events();
 }
@@ -187,24 +223,30 @@ void Badge::move(int left, int top, int bottom) {
 	if (!_view) {
 		return;
 	}
+	const auto &style = st();
 	const auto star = !_emojiStatus
 		&& (_content.badge == BadgeType::Premium
 			|| _content.badge == BadgeType::Verified);
 	const auto fake = !_emojiStatus && !star;
-	const auto skip = fake ? 0 : _st.position.x();
+	const auto skip = fake ? 0 : style.position.x();
 	const auto badgeLeft = left + skip;
 	const auto badgeTop = top
 		+ (star
-			? _st.position.y()
+			? style.position.y()
 			: (bottom - top - _view->height()) / 2);
 	_view->moveToLeft(badgeLeft, badgeTop);
 }
 
+const style::InfoPeerBadge &Badge::st() const {
+	return _overrideSt ? *_overrideSt : _st;
+}
+
 Data::CustomEmojiSizeTag Badge::sizeTag() const {
 	using SizeTag = Data::CustomEmojiSizeTag;
-	return (_st.sizeTag == 2)
+	const auto &style = st();
+	return (style.sizeTag == 2)
 		? SizeTag::Isolated
-		: (_st.sizeTag == 1)
+		: (style.sizeTag == 1)
 		? SizeTag::Large
 		: SizeTag::Normal;
 }
@@ -237,4 +279,18 @@ rpl::producer<Badge::Content> VerifiedContentForPeer(
 	});
 }
 
+rpl::producer<Badge::Content> BotVerifyBadgeForPeer(
+		not_null<PeerData*> peer) {
+	return peer->session().changes().peerFlagsValue(
+		peer,
+		Data::PeerUpdate::Flag::VerifyInfo
+	) | rpl::map([=] {
+		const auto info = peer->botVerifyDetails();
+		return Badge::Content{
+			.badge = info ? BadgeType::BotVerified : BadgeType::None,
+			.emojiStatusId = { info ? info->iconId : DocumentId() },
+		};
+	});
+}
+
 } // namespace Info::Profile
