@@ -887,6 +887,8 @@ void CreditsRow::init() {
 			tr::now,
 			lt_count,
 			_entry.paidMessagesCount)
+		: _entry.postsSearch
+		? tr::lng_credits_box_history_entry_posts_search(tr::now)
 		: ((!_entry.subscriptionUntil.isNull() && !isSpecial)
 			|| (_entry.giftUpgraded && !isSpecial)
 			|| (_entry.giftResale && !isSpecial)
@@ -997,14 +999,9 @@ void CreditsRow::init() {
 		}
 	}
 	if (!_paintUserpicCallback) {
-		_paintUserpicCallback = /*_entry.stargift
-			? Ui::GenerateGiftStickerUserpicCallback(
-				_session,
-				_entry.bareGiftStickerId,
-				_context.repaint)
-			: */!isSpecial
-			? PeerListRow::generatePaintUserpicCallback(false)
-			: Ui::GenerateCreditsPaintUserpicCallback(_entry);
+		_paintUserpicCallback = (isSpecial || _entry.postsSearch)
+			? Ui::GenerateCreditsPaintUserpicCallback(_entry)
+			: PeerListRow::generatePaintUserpicCallback(false);
 	}
 }
 
@@ -1215,13 +1212,13 @@ CreditsController::CreditsController(CreditsDescriptor d)
 		if (data.startsWith(u"ton"_q)) {
 			const auto in = data.split(u":"_q)[1].startsWith(u"in"_q);
 			return std::make_unique<Ui::Text::ShiftedEmoji>(
-				std::make_unique<Ui::Text::StaticCustomEmoji>(
+				std::make_unique<Ui::CustomEmoji::Internal>(
+					data.toString(),
 					Ui::Earn::IconCurrencyColored(
 						st::tonFieldIconSize,
 						in
 							? st::boxTextFgGood->c
-							: st::menuIconAttentionColor->c),
-					data.toString()),
+							: st::menuIconAttentionColor->c)),
 				QPoint(0, st::lineWidth));
 		}
 		const auto desc = DeserializeCreditsRowDescriptionData(
