@@ -11,6 +11,10 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "intro/intro_step.h"
 #include "base/timer.h"
 
+namespace Ui {
+class LinkButton;
+} // namespace Ui
+
 namespace Intro {
 namespace details {
 
@@ -21,6 +25,9 @@ public:
 		not_null<Main::Account*> account,
 		not_null<Data*> data);
 
+	QString accessibilityName() override;
+	QString accessibilityDescription() override;
+
 	void activate() override;
 	void finished() override;
 	void cancelled() override;
@@ -46,6 +53,7 @@ private:
 	void done(const MTPauth_Authorization &authorization);
 
 	rpl::event_stream<QByteArray> _qrCodes;
+	Ui::LinkButton *_skip = nullptr;
 	base::Timer _refreshTimer;
 	mtpRequestId _requestId = 0;
 	bool _forceRefresh = false;
