@@ -116,8 +116,21 @@ Widget::Widget(
 	default: Unexpected("Enter point in Intro::Widget::Widget.");
 	}
 
+	setupStep();
 	fixOrder();
 
+	if (_account->mtp().isTestMode()) {
+		_testModeLabel.create(
+			this,
+			object_ptr<Ui::FlatLabel>(
+				this,
+				u"Test Mode"_q,
+				st::defaultFlatLabel));
+		_testModeLabel->entity()->setTextColorOverride(
+			st::windowSubTextFg->c);
+		_testModeLabel->show(anim::type::instant);
+	}
+
 	Lang::CurrentCloudManager().firstLanguageSuggestion(
 	) | rpl::start_with_next([=] {
 		createLanguageLink();
@@ -129,6 +142,7 @@ Widget::Widget(
 	}, lifetime());
 
 	_back->entity()->setClickedCallback([=] { backRequested(); });
+	_back->entity()->setAccessibleName(tr::lng_go_back(tr::now));
 	_back->hide(anim::type::instant);
 
 	if (_changeLanguage) {
@@ -329,6 +343,32 @@ void Widget::setInnerFocus() {
 	}
 }
 
+void Widget::setupStep() {
+	getStep()->nextButtonStyle(
+	) | rpl::start_with_next([=](const style::RoundButton *st) {
+		const auto nextStyle = st ? st : &st::introNextButton;
+		if (_nextStyle != nextStyle) {
+			_nextStyle = nextStyle;
+			const auto wasShown = _next->toggled();
+			_next.destroy();
+			_next.create(
+				this,
+				object_ptr<Ui::RoundButton>(this, nullptr, *nextStyle));
+			showControls();
+			updateControlsGeometry();
+			_next->toggle(wasShown, anim::type::instant);
+		}
+	}, getStep()->lifetime());
+
+	getStep()->nextButtonFocusRequests() | rpl::start_with_next([=] {
+		if (_next && !_next->isHidden()) {
+			_next->entity()->setFocus(Qt::OtherFocusReason);
+		}
+	}, getStep()->lifetime());
+
+	getStep()->finishInit();
+}
+
 void Widget::historyMove(StackAction action, Animate animate) {
 	Expects(_stepHistory.size() > 1);
 
@@ -350,25 +390,8 @@ void Widget::historyMove(StackAction action, Animate animate) {
 	if (_terms) {
 		hideAndDestroy(std::exchange(_terms, { nullptr }));
 	}
-	{
-		getStep()->nextButtonStyle(
-		) | rpl::start_with_next([=](const style::RoundButton *st) {
-			const auto nextStyle = st ? st : &st::introNextButton;
-			if (_nextStyle != nextStyle) {
-				_nextStyle = nextStyle;
-				const auto wasShown = _next->toggled();
-				_next.destroy();
-				_next.create(
-					this,
-					object_ptr<Ui::RoundButton>(this, nullptr, *nextStyle));
-				showControls();
-				updateControlsGeometry();
-				_next->toggle(wasShown, anim::type::instant);
-			}
-		}, _next->lifetime());
-	}
+	setupStep();
 
-	getStep()->finishInit();
 	getStep()->prepareShowAnimated(wasStep);
 	if (wasStep->hasCover() != getStep()->hasCover()) {
 		_nextTopFrom = wasStep->contentTop() + st::introNextTop;
@@ -392,6 +415,9 @@ void Widget::historyMove(StackAction action, Animate animate) {
 
 	auto stepHasCover = getStep()->hasCover();
 	_settings->toggle(!stepHasCover, anim::type::normal);
+	if (_testModeLabel) {
+		_testModeLabel->toggle(!stepHasCover, anim::type::normal);
+	}
 	if (_update) {
 		_update->toggle(!stepHasCover, anim::type::normal);
 	}
@@ -678,6 +704,9 @@ void Widget::showControls() {
 	_connecting->setForceHidden(false);
 	auto hasCover = getStep()->hasCover();
 	_settings->toggle(!hasCover, anim::type::instant);
+	if (_testModeLabel) {
+		_testModeLabel->toggle(!hasCover, anim::type::instant);
+	}
 	if (_update) {
 		_update->toggle(!hasCover, anim::type::instant);
 	}
@@ -727,6 +756,7 @@ void Widget::hideControls() {
 	_next->hide(anim::type::instant);
 	_connecting->setForceHidden(true);
 	_settings->hide(anim::type::instant);
+	if (_testModeLabel) _testModeLabel->hide(anim::type::instant);
 	if (_update) _update->hide(anim::type::instant);
 	if (_changeLanguage) _changeLanguage->hide(anim::type::instant);
 	if (_terms) _terms->hide(anim::type::instant);
@@ -797,6 +827,13 @@ void Widget::updateControlsGeometry() {
 		getStep()->hasCover() ? st::introCoverHeight : 0,
 		shown);
 	_settings->moveToRight(skip, controlsTop + skip);
+	if (_testModeLabel) {
+		_testModeLabel->moveToRight(
+			skip + _settings->width() + skip,
+			_settings->y()
+				+ (_settings->height()
+				- _testModeLabel->height()) / 2);
+	}
 	if (_update) {
 		_update->moveToRight(
 			skip + _settings->width() + skip,
