@@ -16,6 +16,8 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "boxes/premium_limits_box.h"
 #include "boxes/send_files_box.h"
 #include "boxes/share_box.h" // ShareBoxStyleOverrides
+#include "calls/group/calls_group_call.h"
+#include "calls/group/calls_group_messages.h"
 #include "chat_helpers/compose/compose_show.h"
 #include "chat_helpers/tabbed_selector.h"
 #include "core/file_utilities.h"
@@ -24,7 +26,9 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_changes.h"
 #include "data/data_chat_participant_status.h"
 #include "data/data_document.h"
+#include "data/data_group_call.h"
 #include "data/data_message_reaction_id.h"
+#include "data/data_message_reactions.h"
 #include "data/data_peer_values.h"
 #include "data/data_session.h"
 #include "data/data_user.h"
@@ -35,10 +39,13 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "history/history.h"
 #include "inline_bots/inline_bot_result.h"
 #include "lang/lang_keys.h"
+#include "main/session/send_as_peers.h"
 #include "main/main_session.h"
 #include "media/stories/media_stories_controller.h"
 #include "media/stories/media_stories_stealth.h"
+#include "media/view/media_view_video_stream.h"
 #include "menu/menu_send.h"
+#include "payments/ui/payments_reaction_box.h" // MaxTopPaidDonorsShown
 #include "settings/settings_credits_graphics.h" // DarkCreditsEntryBoxStyle
 #include "storage/localimageloader.h"
 #include "storage/storage_account.h"
@@ -56,19 +63,19 @@ namespace {
 
 [[nodiscard]] rpl::producer<QString> PlaceholderText(
 		const std::shared_ptr<ChatHelpers::Show> &show,
-		rpl::producer<bool> isComment,
+		rpl::producer<ReplyAreaType> type,
 		rpl::producer<int> starsPerMessage) {
 	return rpl::combine(
 		show->session().data().stories().stealthModeValue(),
-		std::move(isComment),
+		std::move(type),
 		std::move(starsPerMessage)
 	) | rpl::map([](
 			Data::StealthMode value,
-			bool isComment,
+			ReplyAreaType type,
 			int starsPerMessage) {
-		return std::tuple(value.enabledTill, isComment, starsPerMessage);
+		return std::tuple(value.enabledTill, type, starsPerMessage);
 	}) | rpl::distinct_until_changed(
-	) | rpl::map([](TimeId till, bool isComment, int starsPerMessage) {
+	) | rpl::map([](TimeId till, ReplyAreaType type, int starsPerMessage) {
 		return rpl::single(
 			rpl::empty
 		) | rpl::then(
@@ -80,7 +87,13 @@ namespace {
 		}) | rpl::then(
 			rpl::single(0)
 		) | rpl::map([=](TimeId left) {
-			return starsPerMessage
+			return (type == ReplyAreaType::VideoStreamComment)
+				? (starsPerMessage
+					? tr::lng_video_stream_comment_paid_ph(
+						lt_count,
+						rpl::single(starsPerMessage * 1.))
+					: tr::lng_video_stream_comment_ph())
+				: starsPerMessage
 				? tr::lng_message_stars_ph(
 					lt_count,
 					rpl::single(starsPerMessage * 1.))
@@ -88,13 +101,37 @@ namespace {
 				? tr::lng_stealth_mode_countdown(
 					lt_left,
 					rpl::single(TimeLeftText(left)))
-				: isComment
+				: (type == ReplyAreaType::Comment)
 				? tr::lng_story_comment_ph()
 				: tr::lng_story_reply_ph();
 		}) | rpl::flatten_latest();
 	}) | rpl::flatten_latest();
 }
 
+[[nodiscard]] ChatHelpers::ComposeFeatures Features(
+		bool videoStream,
+		bool videoStreamManager) {
+	return {
+		.likes = !videoStream,
+		.sendAs = videoStream,
+		.ttlInfo = false,
+		.attachments = !videoStream,
+		.botCommandSend = false,
+		.silentBroadcastToggle = false,
+		.attachBotsMenu = false,
+		.inlineBots = false,
+		.megagroupSet = false,
+		.stickersSettings = false,
+		.openStickerSets = false,
+		.autocompleteHashtags = false,
+		.autocompleteMentions = false,
+		.autocompleteCommands = false,
+		.recordMediaMessage = !videoStream,
+		.editMessageStars = videoStream,
+		.emojiOnlyPanel = videoStream,
+	};
+}
+
 } // namespace
 
 class ReplyArea::Cant final : public Ui::RpWidget {
@@ -141,25 +178,11 @@ ReplyArea::ReplyArea(not_null<Controller*> controller)
 		.stickerOrEmojiChosen = _controller->stickerOrEmojiChosen(),
 		.customPlaceholder = PlaceholderText(
 			_controller->uiShow(),
-			rpl::deferred([=] { return _isComment.value(); }),
+			rpl::deferred([=] { return _type.value(); }),
 			rpl::deferred([=] { return _starsForMessage.value(); })),
 		.voiceCustomCancelText = tr::lng_record_cancel_stories(tr::now),
 		.voiceLockFromBottom = true,
-		.features = {
-			.likes = true,
-			.sendAs = false,
-			.ttlInfo = false,
-			.botCommandSend = false,
-			.silentBroadcastToggle = false,
-			.attachBotsMenu = false,
-			.inlineBots = false,
-			.megagroupSet = false,
-			.stickersSettings = false,
-			.openStickerSets = false,
-			.autocompleteHashtags = false,
-			.autocompleteMentions = false,
-			.autocompleteCommands = false,
-		},
+		.features = Features(false, false),
 	}
 )) {
 	initGeometry();
@@ -217,10 +240,36 @@ bool ReplyArea::sendReaction(const Data::ReactionId &id) {
 }
 
 void ReplyArea::send(Api::SendOptions options) {
+	auto text = _controls->getTextWithAppliedMarkdown();
+	const auto stars = _controls->chosenStarsForMessage();
+	if (const auto stream = _videoStream.get()) {
+		if (stars > 0) {
+			const auto weak = _videoStream;
+			const auto done = [=](Settings::SmallBalanceResult result) {
+				if (result == Settings::SmallBalanceResult::Success
+					|| result == Settings::SmallBalanceResult::Already) {
+					if (const auto strong = weak.get()) {
+						strong->messages()->send(text, stars);
+						_controls->clear();
+					}
+				}
+			};
+			using namespace Settings;
+			MaybeRequestBalanceIncrease(
+				_controller->uiShow(),
+				stars,
+				SmallBalanceVideoStream{ stream->peer()->id },
+				crl::guard(this, done));
+		} else {
+			stream->messages()->send(std::move(text), stars);
+			_controls->clear();
+		}
+		return;
+	}
 	const auto webPageDraft = _controls->webPageDraft();
 
 	auto message = Api::MessageToSend(prepareSendAction(options));
-	message.textWithTags = _controls->getTextWithAppliedMarkdown();
+	message.textWithTags = std::move(text);
 	message.webPage = webPageDraft;
 
 	send(std::move(message));
@@ -580,9 +629,27 @@ void ReplyArea::chooseAttach(
 
 Fn<SendMenu::Details()> ReplyArea::sendMenuDetails() const {
 	return crl::guard(this, [=] {
+		const auto call = _videoStream
+			? _videoStream->lookupReal()
+			: nullptr;
 		return SendMenu::Details{
-			.type = SendMenu::Type::SilentOnly,
-			.effectAllowed = _data.peer && _data.peer->isUser(),
+			.type = (!_data.videoStream
+				? SendMenu::Type::SilentOnly
+				: !call
+				? SendMenu::Type::Disabled
+				: SendMenu::Type::EditCommentPrice),
+			.commentStreamerName = (call
+				? call->peer()->shortName()
+				: QString()),
+			.price = (_data.videoStream
+				? uint64(_controls->chosenStarsForMessage())
+				: std::optional<uint64>()),
+			.commentPriceMin = (call
+				? uint64(call->canManage() ? call->messagesMinPrice() : 0)
+				: std::optional<uint64>()),
+			.effectAllowed = (!_data.videoStream
+				&& _data.peer
+				&& _data.peer->isUser()),
 		};
 	});
 }
@@ -815,49 +882,78 @@ void ReplyArea::show(
 	if (_data == data) {
 		return;
 	}
+	const auto stream = data.videoStream.get();
 	const auto peerChanged = (_data.peer != data.peer);
+	const auto streamChanged = (_data.videoStream.get() != stream);
 	_data = data;
+	if (streamChanged) {
+		const auto manager = stream && stream->canManage();
+		_controls->updateFeatures(Features(stream != nullptr, manager));
+		_controls->setToggleCommentsButton(stream
+			? _controller->commentsStateValue()
+			: nullptr);
+		_controller->setCommentsShownToggles(
+			_controls->commentsShownToggles());
+	}
+	using Controls = HistoryView::ComposeControls;
+	_controls->setStarsReactionCounter(stream
+		? _controller->starsReactionsValue()
+		: nullptr);
+	_controller->setStarsReactionIncrements(
+		_controls->starsReactionIncrements(
+		) | rpl::map([](Controls::StarReactionIncrement increment) {
+			return increment.count;
+		}));
+	_starsForMessage = starsPerMessageValue();
 	if (!peerChanged) {
 		if (_data.peer) {
 			_controls->clear();
 		}
 		return;
-	} else if (const auto peer = _data.peer) {
-		using Flag = Data::PeerUpdate::Flag;
-		_starsForMessage = peer->session().changes().peerFlagsValue(
-			peer,
-			Flag::StarsPerMessage | Flag::FullInfo
-		) | rpl::map([=] {
-			return peer->starsPerMessageChecked();
-		});
-	} else {
-		_starsForMessage = 0;
 	}
 	invalidate_weak_ptrs(&_shownPeerGuard);
 	const auto peer = data.peer;
 	const auto history = peer ? peer->owner().history(peer).get() : nullptr;
 	const auto user = peer->asUser();
-	_isComment = peer->isMegagroup();
-	auto writeRestriction = Data::CanSendAnythingValue(
-		peer
-	) | rpl::map([=](bool can) {
-		using namespace HistoryView::Controls;
-		return peer->session().frozen()
-			? WriteRestriction{ .type = WriteRestrictionType::Frozen }
-			: (can
-			|| !user
-			|| !user->requiresPremiumToWrite()
-			|| user->session().premium())
-			? WriteRestriction()
-			: WriteRestriction{
-				.text = tr::lng_send_non_premium_story(tr::now),
-				.button = tr::lng_send_non_premium_unlock(tr::now),
-				.type = WriteRestrictionType::PremiumRequired,
-			};
-	});
+	_type = peer->isMegagroup()
+		? ReplyAreaType::Comment
+		: ReplyAreaType::Reply;
+	auto writeRestriction = stream
+		? rpl::combine(
+			stream->messagesEnabledValue(),
+			stream->loadedValue()
+		) | rpl::map([=](bool enabled, bool loaded) {
+			using namespace HistoryView::Controls;
+			return !loaded
+				? WriteRestriction{ .type = WriteRestrictionType::Hidden }
+				: enabled
+				? WriteRestriction()
+				: WriteRestriction{
+					.text = tr::lng_video_stream_comments_disabled(tr::now),
+					.type = WriteRestrictionType::Rights,
+				};
+		}) | rpl::type_erased()
+		: Data::CanSendAnythingValue(
+			peer
+		) | rpl::map([=](bool can) {
+			using namespace HistoryView::Controls;
+			return peer->session().frozen()
+				? WriteRestriction{ .type = WriteRestrictionType::Frozen }
+				: (can
+				|| !user
+				|| !user->requiresPremiumToWrite()
+				|| user->session().premium())
+				? WriteRestriction()
+				: WriteRestriction{
+					.text = tr::lng_send_non_premium_story(tr::now),
+					.button = tr::lng_send_non_premium_unlock(tr::now),
+					.type = WriteRestrictionType::PremiumRequired,
+				};
+		});
 	using namespace HistoryView;
 	_controls->setHistory({
 		.history = history,
+		.videoStream = _data.videoStream,
 		.showSlowmodeError = [=] { return showSlowmodeError(); },
 		.sendActionFactory = [=] { return prepareSendAction({}); },
 		.slowmodeSecondsLeft = SlowmodeSecondsLeft(history->peer),
@@ -867,11 +963,15 @@ void ReplyArea::show(
 		) | rpl::map([](const Data::ReactionId &id) {
 			return !id.empty();
 		}),
+		.minStarsCount = (stream
+			? _starsForMessage.value()
+			: rpl::producer<int>()),
 		.writeRestriction = std::move(writeRestriction),
 	});
 	_controls->clear();
 	const auto hidden = peer
-		&& (peer->isBroadcast() || peer->isSelf() || peer->isServiceUser());
+		&& (peer->isBroadcast() || peer->isSelf() || peer->isServiceUser())
+		&& !stream;
 	const auto cant = !peer;
 	if (!hidden && !cant) {
 		_controls->show();
@@ -893,6 +993,32 @@ void ReplyArea::show(
 	}
 }
 
+rpl::producer<int> ReplyArea::starsPerMessageValue() const {
+	if (const auto stream = _data.videoStream.get()) {
+		return rpl::combine(
+			Data::CanManageGroupCallValue(stream->peer()),
+			stream->messagesMinPriceValue()
+		) | rpl::map([=](bool canManage, int price) {
+			return canManage ? 0 : price;
+		});
+	} else if (const auto peer = _data.peer) {
+		using Flag = Data::PeerUpdate::Flag;
+		return peer->session().changes().peerFlagsValue(
+			peer,
+			Flag::StarsPerMessage | Flag::FullInfo
+		) | rpl::map([=] {
+			return peer->starsPerMessageChecked();
+		});
+	}
+	return rpl::single(0);
+}
+
+void ReplyArea::updateVideoStream(not_null<Calls::GroupCall*> videoStream) {
+	_type = ReplyAreaType::VideoStreamComment;
+	_videoStream = videoStream;
+	_controls->setStarsReactionTop(View::TopVideoStreamDonors(videoStream));
+}
+
 bool ReplyArea::showSlowmodeError() {
 	const auto text = [&] {
 		const auto story = _controller->story();
@@ -957,7 +1083,7 @@ void ReplyArea::tryProcessKeyInput(not_null<QKeyEvent*> e) {
 	_controls->tryProcessKeyInput(e);
 }
 
-not_null<Ui::RpWidget*> ReplyArea::likeAnimationTarget() const {
+Ui::RpWidget *ReplyArea::likeAnimationTarget() const {
 	return _controls->likeAnimationTarget();
 }
 
