@@ -383,9 +383,11 @@ MTPInputInvoice Form::inputInvoice() const {
 		}
 		return MTP_inputInvoiceStars(
 			MTP_inputStorePaymentStarsTopup(
+				MTP_flags(0),
 				MTP_long(credits->credits),
 				MTP_string(credits->currency),
-				MTP_long(credits->amount)));
+				MTP_long(credits->amount),
+				MTPInputPeer()));
 	} else if (const auto gift = std::get_if<InvoiceStarGift>(&_id.value)) {
 		using Flag = MTPDinputInvoiceStarGift::Flag;
 		return MTP_inputInvoiceStarGift(
@@ -531,6 +533,7 @@ void Form::requestForm() {
 				.invoice = invoice,
 				.inputInvoice = inputInvoice(),
 				.starGiftLimitedCount = gift ? gift->limitedCount : 0,
+				.starGiftPerUserLimit = gift ? gift->perUserLimit : 0,
 				.starGiftForm = true,
 			};
 			_updates.fire(CreditsPaymentStarted{ .data = formData });
