@@ -11,13 +11,17 @@ enum class HistoryReactionSource : char;
 
 class HistoryItem;
 
+namespace Calls {
+class GroupCall;
+} // namespace Calls
+
 namespace HistoryView {
 class Element;
 } // namespace HistoryView
 
-namespace Ui {
-class Show;
-} // namespace Ui
+namespace Main {
+class SessionShow;
+} // namespace Main
 
 namespace Window {
 class SessionController;
@@ -32,7 +36,14 @@ void TryAddingPaidReaction(
 	HistoryView::Element *view,
 	int count,
 	std::optional<PeerId> shownPeer,
-	std::shared_ptr<Ui::Show> show,
+	std::shared_ptr<Main::SessionShow> show,
+	Fn<void(bool)> finished = nullptr);
+
+void TryAddingPaidReaction(
+	not_null<Calls::GroupCall*> call,
+	int count,
+	std::optional<PeerId> shownPeer,
+	std::shared_ptr<Main::SessionShow> show,
 	Fn<void(bool)> finished = nullptr);
 
 void ShowPaidReactionDetails(
