@@ -881,10 +881,13 @@ void Panel::showCriticalError(const TextWithEntities &text) {
 	_progress = nullptr;
 	_webviewProgress = false;
 	if (!_weakFormSummary || !_weakFormSummary->showCriticalError(text)) {
-		auto error = base::make_unique_q<PaddingWrap<FlatLabel>>(
-			_widget.get(),
+		auto wrap = base::make_unique_q<RpWidget>(_widget.get());
+		const auto raw = wrap.get();
+
+		const auto error = CreateChild<PaddingWrap<FlatLabel>>(
+			raw,
 			object_ptr<FlatLabel>(
-				_widget.get(),
+				raw,
 				rpl::single(text),
 				st::paymentsCriticalError),
 			st::paymentsCriticalErrorPadding);
@@ -898,7 +901,13 @@ void Panel::showCriticalError(const TextWithEntities &text) {
 			_delegate->panelOpenUrl(entity.data);
 			return false;
 		});
-		_widget->showInner(std::move(error));
+
+		raw->widthValue() | rpl::start_with_next([=](int width) {
+			error->resizeToWidth(width);
+			raw->resize(width, error->height());
+		}, raw->lifetime());
+
+		_widget->showInner(std::move(wrap));
 	}
 }
 
