@@ -8,23 +8,30 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "payments/ui/payments_reaction_box.h"
 
 #include "base/qt/qt_compare.h"
+#include "calls/group/ui/calls_group_stars_coloring.h"
 #include "lang/lang_keys.h"
 #include "ui/boxes/boost_box.h" // MakeBoostFeaturesBadge.
 #include "ui/controls/who_reacted_context_action.h"
 #include "ui/effects/premium_bubble.h"
+#include "ui/effects/ministar_particles.h"
 #include "ui/layers/generic_box.h"
 #include "ui/text/text_utilities.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/checkbox.h"
+#include "ui/widgets/labels.h"
 #include "ui/widgets/continuous_sliders.h"
 #include "ui/widgets/popup_menu.h"
 #include "ui/wrap/slide_wrap.h"
+#include "ui/color_int_conversion.h"
 #include "ui/dynamic_image.h"
 #include "ui/painter.h"
 #include "ui/vertical_list.h"
+#include "styles/style_calls.h"
 #include "styles/style_chat.h"
 #include "styles/style_chat_helpers.h"
 #include "styles/style_credits.h"
+#include "styles/style_info.h"
+#include "styles/style_info_levels.h"
 #include "styles/style_layers.h"
 #include "styles/style_media_player.h"
 #include "styles/style_premium.h"
@@ -36,7 +43,8 @@ namespace Settings {
 	not_null<Main::Session*> session,
 	rpl::producer<CreditsAmount> balanceValue,
 	bool rightAlign,
-	rpl::producer<float64> opacityValue = nullptr);
+	rpl::producer<float64> opacityValue = nullptr,
+	bool dark = false);
 } // namespace Settings
 
 namespace Ui {
@@ -57,102 +65,22 @@ struct TopReactorKey {
 		const TopReactorKey &) = default;
 };
 
-struct Discreter {
-	Fn<int(float64)> ratioToValue;
-	Fn<float64(int)> valueToRatio;
-};
-
-[[nodiscard]] Discreter DiscreterForMax(int max) {
-	Expects(max >= 2);
-
-	// 1/8 of width is 1..10
-	// 1/3 of width is 1..100
-	// 2/3 of width is 1..1000
-
-	auto thresholds = base::flat_map<float64, int>();
-	thresholds.emplace(0., 1);
-	if (max <= 40) {
-		thresholds.emplace(1., max);
-	} else if (max <= 300) {
-		thresholds.emplace(1. / 4, 10);
-		thresholds.emplace(1., max);
-	} else if (max <= 600) {
-		thresholds.emplace(1. / 8, 10);
-		thresholds.emplace(1. / 2, 100);
-		thresholds.emplace(1., max);
-	} else if (max <= 1900) {
-		thresholds.emplace(1. / 8, 10);
-		thresholds.emplace(1. / 3, 100);
-		thresholds.emplace(1., max);
-	} else {
-		thresholds.emplace(1. / 8, 10);
-		thresholds.emplace(1. / 3, 100);
-		thresholds.emplace(2. / 3, 1000);
-		thresholds.emplace(1., max);
-	}
-
-	const auto ratioToValue = [=](float64 ratio) {
-		ratio = std::clamp(ratio, 0., 1.);
-		const auto j = thresholds.lower_bound(ratio);
-		if (j == begin(thresholds)) {
-			return 1;
-		}
-		const auto i = j - 1;
-		const auto progress = (ratio - i->first) / (j->first - i->first);
-		const auto value = i->second + (j->second - i->second) * progress;
-		return int(base::SafeRound(value));
-	};
-	const auto valueToRatio = [=](int value) {
-		value = std::clamp(value, 1, max);
-		auto i = begin(thresholds);
-		auto j = i + 1;
-		while (j->second < value) {
-			i = j++;
-		}
-		const auto progress = (value - i->second)
-			/ float64(j->second - i->second);
-		return i->first + (j->first - i->first) * progress;
-	};
-	return {
-		.ratioToValue = ratioToValue,
-		.valueToRatio = valueToRatio,
-	};
-}
-
-void PaidReactionSlider(
-		not_null<VerticalLayout*> container,
-		int current,
-		int max,
-		Fn<void(int)> changed) {
-	Expects(current >= 1 && current <= max);
-
-	const auto slider = container->add(
-		object_ptr<MediaSlider>(container, st::paidReactSlider),
-		st::boxRowPadding + QMargins(0, st::paidReactSliderTop, 0, 0));
-	slider->resize(slider->width(), st::paidReactSlider.seekSize.height());
-
-	const auto discreter = DiscreterForMax(max);
-	slider->setAlwaysDisplayMarker(true);
-	slider->setDirection(ContinuousSlider::Direction::Horizontal);
-	slider->setValue(discreter.valueToRatio(current));
-	slider->setAdjustCallback([=](float64 ratio) {
-		return discreter.valueToRatio(discreter.ratioToValue(ratio));
-	});
-	const auto ratioToValue = discreter.ratioToValue;
-	slider->setChangeProgressCallback([=](float64 value) {
-		changed(ratioToValue(value));
-	});
-	slider->setChangeFinishedCallback([=](float64 value) {
-		changed(ratioToValue(value));
-	});
-}
-
-[[nodiscard]] QImage GenerateBadgeImage(int count) {
+[[nodiscard]] QImage GenerateBadgeImage(
+		const std::vector<Calls::Group::Ui::StarsColoring> &colorings,
+		int count,
+		bool videoStream) {
 	return GenerateSmallBadgeImage(
 		Lang::FormatCountDecimal(count),
 		st::paidReactTopStarIcon,
-		st::creditsBg3->c,
-		st::premiumButtonFg->c);
+		(videoStream
+			? Ui::ColorFromSerialized(
+				Calls::Group::Ui::StarsColoringForCount(
+					colorings,
+					count
+				).bgLight)
+			: st::creditsBg3->c),
+		videoStream ? st::white->c : st::premiumButtonFg->c,
+		videoStream ? &st::groupCallTopReactorBadge : nullptr);
 }
 
 void AddArrowDown(not_null<RpWidget*> widget) {
@@ -182,8 +110,19 @@ void AddArrowDown(not_null<RpWidget*> widget) {
 [[nodiscard]] not_null<RpWidget*> MakeTopReactor(
 		not_null<QWidget*> parent,
 		const PaidReactionTop &data,
-		Fn<void()> selectShownPeer) {
+		int place,
+		const std::vector<Calls::Group::Ui::StarsColoring> &colorings,
+		Fn<void()> selectShownPeer,
+		bool videoStream) {
+	auto top = 0;
+	auto height = st::paidReactTopNameSkip + st::normalFont->height;
+	if (videoStream) {
+		top += st::paidReactCrownSkip;
+		height += top;
+	}
+
 	const auto result = CreateChild<AbstractButton>(parent);
+	result->resize(0, height);
 	result->show();
 	if (data.click && !data.my) {
 		result->setClickedCallback(data.click);
@@ -213,21 +152,45 @@ void AddArrowDown(not_null<RpWidget*> widget) {
 	result->paintRequest() | rpl::start_with_next([=] {
 		auto p = Painter(result);
 		const auto left = (result->width() - st::paidReactTopUserpic) / 2;
-		p.drawImage(left, 0, photo->image(st::paidReactTopUserpic));
+		p.drawImage(left, top, photo->image(st::paidReactTopUserpic));
 
 		if (state->badge.isNull()) {
-			state->badge = GenerateBadgeImage(count);
+			state->badge = GenerateBadgeImage(colorings, count, videoStream);
 		}
 		const auto bwidth = state->badge.width()
 			/ state->badge.devicePixelRatio();
 		p.drawImage(
 			(result->width() - bwidth) / 2,
-			st::paidReactTopBadgeSkip,
+			top + st::paidReactTopBadgeSkip,
 			state->badge);
 
-		p.setPen(st::windowFg);
+		if (videoStream) {
+			const auto bg = Calls::Group::Ui::StarsColoringForCount(
+				colorings,
+				count
+			).bgLight;
+			const auto &icon = st::paidReactCrown;
+			const auto left = (result->width() - icon.width()) / 2;
+			const auto shift = st::paidReactCrownOutline;
+			const auto outline = st::groupCallMembersBg->c;
+			icon.paint(p, left - shift, shift, result->width(), outline);
+			icon.paint(p, left + shift, shift, result->width(), outline);
+			icon.paint(p, left, 0, result->width(), bg);
+
+			const auto top = st::paidReactCrownTop;
+			p.setPen(st::white);
+			p.setFont(st::levelStyle.font);
+			p.drawText(
+				QRect(left, top, icon.width(), icon.height()),
+				QString::number(place),
+				style::al_top);
+
+			p.setPen(st::groupCallMembersFg);
+		} else {
+			p.setPen(st::windowFg);
+		}
 		const auto skip = st::normalFont->spacew;
-		const auto nameTop = st::paidReactTopNameSkip;
+		const auto nameTop = top + st::paidReactTopNameSkip;
 		const auto available = result->width() - skip * 2;
 		state->name.draw(p, skip, nameTop, available, style::al_top);
 	}, result->lifetime());
@@ -285,22 +248,27 @@ void SelectShownPeer(
 void FillTopReactors(
 		not_null<VerticalLayout*> container,
 		std::vector<PaidReactionTop> top,
+		const std::vector<Calls::Group::Ui::StarsColoring> &colorings,
 		rpl::producer<int> chosen,
 		rpl::producer<uint64> shownPeer,
-		Fn<void(uint64)> changeShownPeer) {
-	container->add(
-		MakeBoostFeaturesBadge(
-			container,
-			tr::lng_paid_react_top_title(),
-			[](QRect) { return st::creditsBg3->b; }),
-		st::boxRowPadding + st::paidReactTopTitleMargin,
-		style::al_top);
-
-	const auto height = st::paidReactTopNameSkip + st::normalFont->height;
+		Fn<void(uint64)> changeShownPeer,
+		bool videoStream) {
+	const auto badge = videoStream
+		? nullptr
+		: container->add(
+			object_ptr<SlideWrap<RpWidget>>(
+				container,
+				MakeBoostFeaturesBadge(
+					container,
+					tr::lng_paid_react_top_title(),
+					[](QRect) { return st::creditsBg3->b; }),
+				st::paidReactTopTitleMargin),
+			st::boxRowPadding,
+			style::al_top);
 	const auto wrap = container->add(
 		object_ptr<SlideWrap<FixedHeightWidget>>(
 			container,
-			object_ptr<FixedHeightWidget>(container, height),
+			object_ptr<FixedHeightWidget>(container, 0),
 			st::paidReactTopMargin));
 	const auto parent = wrap->entity();
 	using Key = TopReactorKey;
@@ -357,12 +325,19 @@ void FillTopReactors(
 				barePeerId,
 				changeShownPeer); };
 		if (list.empty()) {
+			if (badge) {
+				badge->hide(anim::type::normal);
+			}
 			wrap->hide(anim::type::normal);
 		} else {
+			if (badge) {
+				badge->show(anim::type::normal);
+			}
 			for (const auto &widget : state->widgets) {
 				widget->hide();
 			}
 			state->widgets.clear();
+			auto index = 0;
 			for (const auto &entry : list) {
 				const auto key = Key{
 					.photo = entry.photo,
@@ -372,7 +347,13 @@ void FillTopReactors(
 				const auto i = state->cache.find(key);
 				const auto widget = (i != end(state->cache))
 					? i->second
-					: MakeTopReactor(parent, entry, selectShownPeer);
+					: MakeTopReactor(
+						parent,
+						entry,
+						++index,
+						colorings,
+						selectShownPeer,
+						videoStream);
 				state->widgets.push_back(widget);
 				widget->show();
 			}
@@ -386,12 +367,18 @@ void FillTopReactors(
 
 		state->updated.fire({});
 	}, wrap->lifetime());
+	if (badge) {
+		badge->finishAnimating();
+	}
 	wrap->finishAnimating();
 
 	rpl::combine(
 		state->updated.events_starting_with({}),
 		wrap->widthValue()
 	) | rpl::start_with_next([=](auto, int width) {
+		if (!state->widgets.empty()) {
+			parent->resize(parent->width(), state->widgets.back()->height());
+		}
 		const auto single = width / 4;
 		if (single <= st::paidReactTopUserpic) {
 			return;
@@ -399,12 +386,73 @@ void FillTopReactors(
 		const auto count = int(state->widgets.size());
 		auto left = (width - single * count) / 2;
 		for (const auto &widget : state->widgets) {
-			widget->setGeometry(left, 0, single, height);
+			widget->setGeometry(left, 0, single, widget->height());
 			left += single;
 		}
 	}, wrap->lifetime());
 }
 
+[[nodiscard]] not_null<RpWidget*> MakeStarSelectInfoBlock(
+		not_null<RpWidget*> parent,
+		rpl::producer<TextWithEntities> title,
+		rpl::producer<QString> subtext,
+		Fn<void()> click,
+		Text::MarkedContext context,
+		bool dark) {
+	const auto result = CreateChild<AbstractButton>(parent);
+
+	const auto titleHeight = st::starSelectInfoTitle.style.font->height;
+	const auto subtextHeight = st::starSelectInfoSubtext.style.font->height;
+	const auto height = titleHeight + subtextHeight;
+
+	result->paintRequest() | rpl::start_with_next([=] {
+		auto p = QPainter(result);
+		auto hq = PainterHighQualityEnabler(p);
+		p.setPen(Qt::NoPen);
+		p.setBrush(dark ? st::groupCallMembersBgOver : st::windowBgOver);
+		const auto radius = st::boxRadius;
+		p.drawRoundedRect(result->rect(), radius, radius);
+	}, result->lifetime());
+
+	result->resize(
+		result->width(),
+		QSize(height, height).grownBy(st::starSelectInfoPadding).height());
+
+	const auto titleLabel = CreateChild<FlatLabel>(
+		result,
+		std::move(title),
+		dark ? st::videoStreamInfoTitle : st::starSelectInfoTitle,
+		st::defaultPopupMenu,
+		context);
+	const auto subtextLabel = CreateChild<FlatLabel>(
+		result,
+		std::move(subtext),
+		dark ? st::videoStreamInfoSubtext : st::starSelectInfoSubtext);
+
+	if (click) {
+		result->setClickedCallback(std::move(click));
+		titleLabel->setAttribute(Qt::WA_TransparentForMouseEvents);
+		subtextLabel->setAttribute(Qt::WA_TransparentForMouseEvents);
+	} else {
+		result->setAttribute(Qt::WA_TransparentForMouseEvents);
+	}
+
+	rpl::combine(
+		result->widthValue(),
+		titleLabel->widthValue(),
+		subtextLabel->widthValue()
+	) | rpl::start_with_next([=](int width, int titlew, int subtextw) {
+		const auto padding = st::starSelectInfoPadding;
+		titleLabel->moveToLeft((width - titlew) / 2, padding.top(), width);
+		subtextLabel->moveToLeft(
+			(width - subtextw) / 2,
+			padding.top() + titleHeight,
+			width);
+	}, result->lifetime());
+
+	return result;
+}
+
 } // namespace
 
 void PaidReactionsBox(
@@ -412,11 +460,22 @@ void PaidReactionsBox(
 		PaidReactionBoxArgs &&args) {
 	Expects(!args.top.empty());
 
-	args.max = std::max(args.max, 2);
-	args.chosen = std::clamp(args.chosen, 1, args.max);
+	const auto dark = args.dark;
+	args.min = std::max(args.min, 1);
+	args.max = std::max({
+		args.min + 1,
+		args.max,
+		args.explicitlyAllowed,
+		args.chosen,
+	});
+
+	const auto allowed = args.explicitlyAllowed;
+	args.chosen = (allowed && args.chosen == allowed)
+		? allowed
+		: std::clamp(args.chosen, args.min, args.max);
 
 	box->setWidth(st::boxWideWidth);
-	box->setStyle(st::paidReactBox);
+	box->setStyle(dark ? st::darkEditStarsBox : st::paidReactBox);
 	box->setNoContentMargin(true);
 
 	struct State {
@@ -431,6 +490,10 @@ void PaidReactionsBox(
 		state->chosen = count;
 	};
 
+	const auto colorings = args.colorings;
+	const auto videoStreamChoosing = args.videoStreamChoosing;
+	const auto videoStreamSending = args.videoStreamSending;
+	const auto videoStream = videoStreamChoosing || videoStreamSending;
 	const auto initialShownPeer = ranges::find(
 		args.top,
 		true,
@@ -445,61 +508,95 @@ void PaidReactionsBox(
 	const auto content = box->verticalLayout();
 	AddSkip(content, st::boxTitleClose.height + st::paidReactBubbleTop);
 
-	const auto valueToRatio = DiscreterForMax(args.max).valueToRatio;
-	auto bubbleRowState = state->chosen.value() | rpl::map([=](int value) {
-		const auto full = st::boxWideWidth
-			- st::boxRowPadding.left()
-			- st::boxRowPadding.right();
-		const auto marker = st::paidReactSlider.seekSize.width();
-		const auto start = marker / 2;
-		const auto inner = full - marker;
-		const auto correct = start + inner * valueToRatio(value);
-		return Premium::BubbleRowState{
-			.counter = value,
-			.ratio = correct / full,
-		};
-	});
-	Premium::AddBubbleRow(
+	const auto activeFgOverride = [=](int count) {
+		const auto coloring = Calls::Group::Ui::StarsColoringForCount(
+			colorings,
+			count);
+		return Ui::ColorFromSerialized(coloring.bgLight);
+	};
+	AddStarSelectBubble(
 		content,
-		st::boostBubble,
 		BoxShowFinishes(box),
-		std::move(bubbleRowState),
-		Premium::BubbleType::Credits,
-		nullptr,
-		&st::paidReactBubbleIcon,
-		st::boxRowPadding);
+		state->chosen.value(),
+		args.max,
+		videoStream ? activeFgOverride : Fn<QColor(int)>());
 
 	const auto already = ranges::find(
 		args.top,
 		true,
 		&PaidReactionTop::my)->count;
-	PaidReactionSlider(content, args.chosen, args.max, changed);
+	PaidReactionSlider(
+		content,
+		(dark ? st::darkEditStarsSlider : st::paidReactSlider),
+		args.min,
+		args.explicitlyAllowed,
+		rpl::single(args.chosen),
+		args.max,
+		changed,
+		videoStream ? activeFgOverride : Fn<QColor(int)>());
 
-	box->addTopButton(st::boxTitleClose, [=] { box->closeBox(); });
+	box->addTopButton(
+		dark ? st::darkEditStarsClose : st::boxTitleClose,
+		[=] { box->closeBox(); });
+
+	const auto addTopReactors = [&] {
+		FillTopReactors(
+			content,
+			std::move(args.top),
+			colorings,
+			state->chosen.value(),
+			state->shownPeer.value(),
+			[=](uint64 barePeerId) {
+				state->shownPeer = state->savedShownPeer = barePeerId;
+			},
+			videoStream);
+	};
+
+	if (videoStreamChoosing) {
+		using namespace Calls::Group::Ui;
+		box->addRow(
+			VideoStreamStarsLevel(box, colorings, state->chosen.value()),
+			st::boxRowPadding + QMargins(0, st::paidReactTitleSkip, 0, 0));
+	} else if (videoStreamSending) {
+		addTopReactors();
+	}
 
 	box->addRow(
 		object_ptr<FlatLabel>(
 			box,
-			tr::lng_paid_react_title(),
-			st::boostCenteredTitle),
+			(videoStreamChoosing
+				? tr::lng_paid_comment_title()
+				: videoStreamSending
+				? tr::lng_paid_reaction_title()
+				: tr::lng_paid_react_title()),
+			dark ? st::darkEditStarsCenteredTitle : st::boostCenteredTitle),
 		st::boxRowPadding + QMargins(0, st::paidReactTitleSkip, 0, 0),
 		style::al_top);
+
 	const auto labelWrap = box->addRow(
 		object_ptr<RpWidget>(box),
 		(st::boxRowPadding
 			+ QMargins(0, st::lineWidth, 0, st::boostBottomSkip)));
 	const auto label = CreateChild<FlatLabel>(
 		labelWrap,
-		(already
+		(videoStream
+			? (videoStreamChoosing
+				? tr::lng_paid_comment_about
+				: tr::lng_paid_reaction_about)(
+					lt_name,
+					rpl::single(Text::Bold(args.name)),
+					Text::RichLangValue)
+			: already
 			? tr::lng_paid_react_already(
 				lt_count,
 				rpl::single(already) | tr::to_count(),
 				Text::RichLangValue)
 			: tr::lng_paid_react_about(
 				lt_channel,
-				rpl::single(Text::Bold(args.channel)),
+				rpl::single(Text::Bold(args.name)),
 				Text::RichLangValue)),
-		st::boostText);
+		dark ? st::darkEditStarsText : st::boostText);
+	label->setTryMakeSimilarLines(true);
 	labelWrap->widthValue() | rpl::start_with_next([=](int width) {
 		label->resizeToWidth(width);
 	}, label->lifetime());
@@ -510,25 +607,40 @@ void PaidReactionsBox(
 		label->moveToLeft(0, skip);
 	}, label->lifetime());
 
-	FillTopReactors(
-		content,
-		std::move(args.top),
-		state->chosen.value(),
-		state->shownPeer.value(),
-		[=](uint64 barePeerId) {
-			state->shownPeer = state->savedShownPeer = barePeerId;
-		});
+	if (!videoStream) {
+		addTopReactors();
 
-	const auto named = box->addRow(
-		object_ptr<Checkbox>(
-			box,
-			tr::lng_paid_react_show_in_top(tr::now),
-			state->shownPeer.current() != 0),
-		style::al_top);
-	named->checkedValue(
-	) | rpl::start_with_next([=](bool show) {
-		state->shownPeer = show ? state->savedShownPeer : 0;
-	}, named->lifetime());
+		const auto skip = st::defaultCheckbox.margin.bottom();
+		const auto named = box->addRow(
+			object_ptr<Checkbox>(
+				box,
+				tr::lng_paid_react_show_in_top(tr::now),
+				state->shownPeer.current() != 0,
+				st::paidReactBoxCheckbox),
+			st::boxRowPadding + QMargins(0, 0, 0, skip),
+			style::al_top);
+		named->checkedValue(
+		) | rpl::start_with_next([=](bool show) {
+			state->shownPeer = show ? state->savedShownPeer : 0;
+		}, named->lifetime());
+	}
+
+	AddSkip(content);
+	AddSkip(content);
+
+	AddDividerText(
+		content,
+		tr::lng_paid_react_agree(
+			lt_link,
+			rpl::combine(
+				tr::lng_paid_react_agree_link(),
+				tr::lng_group_invite_subscription_about_url()
+			) | rpl::map([](const QString &text, const QString &url) {
+				return Ui::Text::Link(text, url);
+			}),
+			Ui::Text::RichLangValue),
+		st::defaultBoxDividerLabelPadding,
+		dark ? st::groupCallDividerLabel : st::defaultDividerLabel);
 
 	const auto button = box->addButton(rpl::single(QString()), [=] {
 		args.send(state->chosen.current(), state->shownPeer.current());
@@ -540,58 +652,23 @@ void PaidReactionsBox(
 		args.send(0, state->shownPeer.current());
 	}, box->lifetime());
 
-	{
-		const auto buttonLabel = CreateChild<FlatLabel>(
-			button,
-			rpl::single(QString()),
-			st::creditsBoxButtonLabel);
-		args.submit(
-			state->chosen.value()
-		) | rpl::start_with_next([=](const TextWithEntities &text) {
-			buttonLabel->setMarkedText(text);
-		}, buttonLabel->lifetime());
-		buttonLabel->setTextColorOverride(
-			box->getDelegate()->style().button.textFg->c);
-		button->sizeValue(
-		) | rpl::start_with_next([=](const QSize &size) {
-			buttonLabel->moveToLeft(
-				(size.width() - buttonLabel->width()) / 2,
-				(size.height() - buttonLabel->height()) / 2);
-		}, buttonLabel->lifetime());
-		buttonLabel->setAttribute(Qt::WA_TransparentForMouseEvents);
-	}
+	button->setText(args.submit(state->chosen.value()));
 
-	box->widthValue(
-	) | rpl::start_with_next([=](int width) {
-		const auto &padding = st::paidReactBox.buttonPadding;
-		button->resizeToWidth(width
-			- padding.left()
-			- padding.right());
-		button->moveToLeft(padding.left(), button->y());
-	}, button->lifetime());
-
-	{
-		const auto balance = Settings::AddBalanceWidget(
-			content,
-			args.session,
-			std::move(args.balanceValue),
-			false);
-		rpl::combine(
-			balance->sizeValue(),
-			box->widthValue()
-		) | rpl::start_with_next([=] {
-			balance->moveToLeft(
-				st::creditsHistoryRightSkip * 2,
-				st::creditsHistoryRightSkip);
-			balance->update();
-		}, balance->lifetime());
-	}
+	AddStarSelectBalance(
+		box,
+		args.session,
+		std::move(args.balanceValue),
+		dark);
 }
 
 object_ptr<BoxContent> MakePaidReactionBox(PaidReactionBoxArgs &&args) {
 	return Box(PaidReactionsBox, std::move(args));
 }
 
+int MaxTopPaidDonorsShown() {
+	return kMaxTopPaidShown;
+}
+
 QImage GenerateSmallBadgeImage(
 		QString text,
 		const style::icon &icon,
@@ -653,4 +730,295 @@ QImage GenerateSmallBadgeImage(
 	return result;
 }
 
+StarSelectDiscreter StarSelectDiscreterForMax(int max) {
+	Expects(max >= 2);
+
+	// 1/8 of width is 1..10
+	// 1/3 of width is 1..100
+	// 2/3 of width is 1..1000
+
+	auto thresholds = base::flat_map<float64, int>();
+	thresholds.emplace(0., 1);
+	if (max <= 40) {
+		thresholds.emplace(1., max);
+	} else if (max <= 300) {
+		thresholds.emplace(1. / 4, 10);
+		thresholds.emplace(1., max);
+	} else if (max <= 600) {
+		thresholds.emplace(1. / 8, 10);
+		thresholds.emplace(1. / 2, 100);
+		thresholds.emplace(1., max);
+	} else if (max <= 1900) {
+		thresholds.emplace(1. / 8, 10);
+		thresholds.emplace(1. / 3, 100);
+		thresholds.emplace(1., max);
+	} else if (max <= 10000) {
+		thresholds.emplace(1. / 8, 10);
+		thresholds.emplace(1. / 3, 100);
+		thresholds.emplace(2. / 3, 1000);
+		thresholds.emplace(1., max);
+	} else {
+		thresholds.emplace(1. / 10, 10);
+		thresholds.emplace(1. / 6, 100);
+		thresholds.emplace(1. / 3, 1000);
+		thresholds.emplace(1., max);
+	}
+
+	const auto ratioToValue = [=](float64 ratio) {
+		ratio = std::clamp(ratio, 0., 1.);
+		const auto j = thresholds.lower_bound(ratio);
+		if (j == begin(thresholds)) {
+			return 1;
+		}
+		const auto i = j - 1;
+		const auto progress = (ratio - i->first) / (j->first - i->first);
+		const auto value = i->second + (j->second - i->second) * progress;
+		return int(base::SafeRound(value));
+	};
+	const auto valueToRatio = [=](int value) {
+		value = std::clamp(value, 1, max);
+		auto i = begin(thresholds);
+		auto j = i + 1;
+		while (j->second < value) {
+			i = j++;
+		}
+		const auto progress = (value - i->second)
+			/ float64(j->second - i->second);
+		return i->first + (j->first - i->first) * progress;
+	};
+	return {
+		.ratioToValue = ratioToValue,
+		.valueToRatio = valueToRatio,
+	};
+}
+
+void PaidReactionSlider(
+		not_null<VerticalLayout*> container,
+		const style::MediaSlider &st,
+		int min,
+		int explicitlyAllowed,
+		rpl::producer<int> current,
+		int max,
+		Fn<void(int)> changed,
+		Fn<QColor(int)> activeFgOverride) {
+	Expects(explicitlyAllowed <= max);
+
+	if (!explicitlyAllowed) {
+		explicitlyAllowed = min;
+	}
+	const auto slider = container->add(
+		object_ptr<MediaSlider>(container, st),
+		st::boxRowPadding + QMargins(0, st::paidReactSliderTop, 0, 0));
+	slider->resize(slider->width(), st::paidReactSlider.seekSize.height());
+
+	const auto update = [=](int count) {
+		if (activeFgOverride) {
+			const auto color = activeFgOverride(count);
+			slider->setColorOverrides({
+				.activeBg = color,
+				.activeBorder = color,
+				.seekFg = st::groupCallMembersFg->c,
+				.seekBorder = color,
+				.inactiveBorder = Qt::transparent,
+			});
+		}
+	};
+
+	const auto discreter = StarSelectDiscreterForMax(max);
+	slider->setAlwaysDisplayMarker(true);
+	slider->setDirection(ContinuousSlider::Direction::Horizontal);
+
+	const auto ratioToValue = [=](float64 ratio) {
+		const auto value = discreter.ratioToValue(ratio);
+		return (value <= explicitlyAllowed && explicitlyAllowed < min)
+			? explicitlyAllowed
+			: std::max(value, min);
+	};
+
+	std::move(current) | rpl::start_with_next([=](int value) {
+		value = std::clamp(value, 1, max);
+		if (discreter.ratioToValue(slider->value()) != value) {
+			slider->setValue(discreter.valueToRatio(value));
+			update(value);
+		}
+	}, slider->lifetime());
+
+	slider->setAdjustCallback([=](float64 ratio) {
+		return discreter.valueToRatio(ratioToValue(ratio));
+	});
+	const auto callback = [=](float64 ratio) {
+		const auto value = ratioToValue(ratio);
+		update(value);
+		changed(value);
+	};
+	slider->setChangeProgressCallback(callback);
+	slider->setChangeFinishedCallback(callback);
+
+
+
+	struct State {
+		StarParticles particles = StarParticles(
+			StarParticles::Type::Right,
+			200,
+			st::lineWidth * 7);
+		Ui::Animations::Basic animation;
+	};
+	const auto state = slider->lifetime().make_state<State>();
+
+	const auto stars = Ui::CreateChild<Ui::RpWidget>(slider->parentWidget());
+	stars->show();
+	stars->raise();
+	slider->geometryValue() | rpl::start_with_next([=](QRect rect) {
+		stars->setGeometry(rect);
+	}, stars->lifetime());
+
+	state->animation.init([=] { stars->update(); });
+	stars->setAttribute(Qt::WA_TransparentForMouseEvents);
+
+	const auto seekSize = st::paidReactSlider.seekSize.width();
+	const auto seekRadius = seekSize / 2.;
+	stars->paintRequest() | rpl::start_with_next([=] {
+		if (!state->animation.animating()) {
+			state->animation.start();
+		}
+		auto p = QPainter(stars);
+		auto hq = PainterHighQualityEnabler(p);
+		const auto progress = slider->value();
+		const auto rect = stars->rect();
+		const auto availableWidth = rect.width() - seekSize;
+		const auto seekCenter = seekRadius + availableWidth * progress;
+
+		state->particles.setSpeed(.1 + progress * .3);
+		state->particles.setVisible(.25 + .65 * progress);
+
+		auto fullPath = QPainterPath();
+		fullPath.addRoundedRect(QRectF(rect), seekRadius, seekRadius);
+		auto circlePath = QPainterPath();
+		circlePath.addEllipse(
+			QPointF(seekCenter, rect.height() / 2.),
+			seekRadius,
+			seekRadius);
+		auto rightRect = QPainterPath();
+		rightRect.addRect(
+			QRectF(seekCenter, 0, rect.width() - seekCenter, rect.height()));
+
+		p.setClipPath(fullPath.subtracted(circlePath));
+		state->particles.setColor(Qt::white);
+		state->particles.paint(p, rect, crl::now());
+		p.setClipping(false);
+
+		p.setClipPath(fullPath.intersected(circlePath.united(rightRect)));
+		state->particles.setColor(activeFgOverride
+			? st::groupCallMemberInactiveIcon->c
+			: st::creditsBg3->c);
+		state->particles.paint(p, rect, crl::now());
+	}, stars->lifetime());
+}
+
+void AddStarSelectBalance(
+		not_null<GenericBox*> box,
+		not_null<Main::Session*> session,
+		rpl::producer<CreditsAmount> balanceValue,
+		bool dark) {
+	const auto balance = Settings::AddBalanceWidget(
+		box->verticalLayout(),
+		session,
+		std::move(balanceValue),
+		false,
+		nullptr,
+		dark);
+	rpl::combine(
+		balance->sizeValue(),
+		box->widthValue()
+	) | rpl::start_with_next([=] {
+		balance->moveToLeft(
+			st::creditsHistoryRightSkip * 2,
+			st::creditsHistoryRightSkip);
+		balance->update();
+	}, balance->lifetime());
+}
+
+not_null<Premium::BubbleWidget*> AddStarSelectBubble(
+		not_null<VerticalLayout*> container,
+		rpl::producer<> showFinishes,
+		rpl::producer<int> value,
+		int max,
+		Fn<QColor(int)> activeFgOverride) {
+	const auto valueToRatio = StarSelectDiscreterForMax(max).valueToRatio;
+	auto bubbleRowState = rpl::duplicate(value) | rpl::map([=](int value) {
+		const auto full = st::boxWideWidth
+			- st::boxRowPadding.left()
+			- st::boxRowPadding.right();
+		const auto marker = st::paidReactSlider.seekSize.width();
+		const auto start = marker / 2;
+		const auto inner = full - marker;
+		const auto correct = start + inner * valueToRatio(value);
+		return Premium::BubbleRowState{
+			.counter = value,
+			.ratio = correct / full,
+		};
+	});
+
+	const auto bubble = Premium::AddBubbleRow(
+		container,
+		st::boostBubble,
+		std::move(showFinishes),
+		std::move(bubbleRowState),
+		Premium::BubbleType::Credits,
+		nullptr,
+		&st::paidReactBubbleIcon,
+		st::boxRowPadding);
+	bubble->show();
+	if (activeFgOverride) {
+		std::move(value) | rpl::start_with_next([=](int count) {
+			bubble->setBrushOverride(activeFgOverride(count));
+		}, bubble->lifetime());
+	}
+	return bubble;
+}
+
+object_ptr<RpWidget> MakeStarSelectInfoBlocks(
+		not_null<RpWidget*> parent,
+		std::vector<StarSelectInfoBlock> blocks,
+		Text::MarkedContext context,
+		bool dark) {
+	Expects(!blocks.empty());
+
+	auto result = object_ptr<RpWidget>(parent.get());
+	const auto raw = result.data();
+
+	struct State {
+		std::vector<not_null<RpWidget*>> blocks;
+	};
+	const auto state = raw->lifetime().make_state<State>();
+
+	for (auto &info : blocks) {
+		state->blocks.push_back(MakeStarSelectInfoBlock(
+			raw,
+			std::move(info.title),
+			std::move(info.subtext),
+			std::move(info.click),
+			context,
+			dark));
+	}
+	raw->resize(raw->width(), state->blocks.front()->height());
+	raw->widthValue() | rpl::start_with_next([=](int width) {
+		const auto count = int(state->blocks.size());
+		const auto skip = (st::boxRowPadding.left() / 2);
+		const auto single = (width - skip * (count - 1)) / float64(count);
+		if (single < 1.) {
+			return;
+		}
+		auto x = 0.;
+		const auto w = int(base::SafeRound(single));
+		for (const auto &block : state->blocks) {
+			block->resizeToWidth(w);
+			block->moveToLeft(int(base::SafeRound(x)), 0);
+			x += single + skip;
+		}
+	}, raw->lifetime());
+
+	return result;
+}
+
 } // namespace Ui
