@@ -404,9 +404,9 @@ void Business::setStepDataReference(std::any &data) {
 
 void Business::setupSwipeBack() {
 	using namespace Ui::Controls;
-	
+
 	auto swipeBackData = lifetime().make_state<SwipeBackResult>();
-	
+
 	auto update = [=](SwipeContextData data) {
 		if (data.translation > 0) {
 			if (!swipeBackData->callback) {
@@ -425,7 +425,7 @@ void Business::setupSwipeBack() {
 			(*swipeBackData) = {};
 		}
 	};
-	
+
 	auto init = [=](int, Qt::LayoutDirection direction) {
 		return (direction == Qt::RightToLeft)
 			? DefaultSwipeBackHandlerFinishData([=] {
@@ -433,7 +433,7 @@ void Business::setupSwipeBack() {
 			})
 			: SwipeHandlerFinishData();
 	};
-	
+
 	SetupSwipeHandler({
 		.widget = this,
 		.scroll = v::null,
@@ -585,8 +585,7 @@ void Business::setupContent() {
 						}),
 						Ui::Text::RichLangValue),
 					st::boxDividerLabel),
-				st::defaultBoxDividerLabelPadding,
-				RectPart::Top | RectPart::Bottom));
+				st::defaultBoxDividerLabelPadding));
 		}
 
 		const auto api = inner->lifetime().make_state<Api::SponsoredToggle>(
