@@ -73,17 +73,17 @@ using SectionCustomTopBarData = Info::Settings::SectionCustomTopBarData;
 [[nodiscard]] Data::PremiumSubscriptionOptions SubscriptionOptionsForRows(
 		Data::PremiumSubscriptionOptions result) {
 	for (auto &option : result) {
-		const auto total = option.costTotal;
+		const auto perYear = option.costPerYear;
 		const auto perMonth = option.costPerMonth;
 
-		option.costTotal = tr::lng_premium_gift_per(
+		option.costPerYear = tr::lng_premium_gift_per(
 			tr::now,
 			lt_cost,
 			perMonth);
 		option.costPerMonth = tr::lng_premium_subscribe_total(
 			tr::now,
 			lt_cost,
-			total);
+			perYear);
 
 		if (option.duration == tr::lng_months(tr::now, lt_count, 1)) {
 			option.costPerMonth = QString();
@@ -110,7 +110,7 @@ namespace Gift {
 
 struct Data {
 	PeerId peerId;
-	int months = 0;
+	int days = 0;
 	bool me = false;
 
 	explicit operator bool() const {
@@ -121,7 +121,7 @@ struct Data {
 [[nodiscard]] QString Serialize(const Data &gift) {
 	return QString::number(gift.peerId.value)
 		+ ':'
-		+ QString::number(gift.months)
+		+ QString::number(gift.days)
 		+ ':'
 		+ QString::number(gift.me ? 1 : 0);
 }
@@ -133,7 +133,7 @@ struct Data {
 	}
 	return {
 		.peerId = PeerId(components[0].toULongLong()),
-		.months = components[1].toInt(),
+		.days = components[1].toInt(),
 		.me = (components[2].toInt() == 1),
 	};
 }
@@ -284,6 +284,15 @@ using Order = std::vector<QString>;
 				PremiumFeature::Wallpapers,
 			},
 		},
+		{
+			u"peer_colors"_q,
+			Entry{
+				&st::settingsPremiumIconPeerColors,
+				tr::lng_premium_summary_subtitle_peer_colors(),
+				tr::lng_premium_summary_about_peer_colors(),
+				PremiumFeature::PeerColors,
+			},
+		},
 		{
 			u"stories"_q,
 			Entry{
@@ -1087,9 +1096,9 @@ void Premium::setupSubscriptionOptions(
 
 void Premium::setupSwipeBack() {
 	using namespace Ui::Controls;
-	
+
 	auto swipeBackData = lifetime().make_state<SwipeBackResult>();
-	
+
 	auto update = [=](SwipeContextData data) {
 		if (data.translation > 0) {
 			if (!swipeBackData->callback) {
@@ -1108,7 +1117,7 @@ void Premium::setupSwipeBack() {
 			(*swipeBackData) = {};
 		}
 	};
-	
+
 	auto init = [=](int, Qt::LayoutDirection direction) {
 		return (direction == Qt::RightToLeft)
 			? DefaultSwipeBackHandlerFinishData([=] {
@@ -1116,7 +1125,7 @@ void Premium::setupSwipeBack() {
 			})
 			: SwipeHandlerFinishData();
 	};
-	
+
 	SetupSwipeHandler({
 		.widget = this,
 		.scroll = v::null,
@@ -1177,11 +1186,16 @@ base::weak_qptr<Ui::RpWidget> Premium::createPinnedToTop(
 		if (gift) {
 			auto &data = _controller->session().data();
 			if (const auto peer = data.peer(gift.peerId)) {
+				const auto months = gift.days / 30;
 				return (gift.me
-					? tr::lng_premium_summary_subtitle_gift_me
-					: tr::lng_premium_summary_subtitle_gift)(
+					? (months
+						? tr::lng_premium_summary_subtitle_gift_me
+						: tr::lng_premium_summary_subtitle_gift_days_me)
+					: (months
+						? tr::lng_premium_summary_subtitle_gift
+						: tr::lng_premium_summary_subtitle_gift_days))(
 						lt_count,
-						rpl::single(float64(gift.months)),
+						rpl::single(float64(months ? months : gift.days)),
 						lt_user,
 						rpl::single(Ui::Text::Bold(peer->name())),
 						Ui::Text::RichLangValue);
@@ -1501,9 +1515,9 @@ void ShowPremium(
 void ShowGiftPremium(
 		not_null<Window::SessionController*> controller,
 		not_null<PeerData*> peer,
-		int months,
+		int days,
 		bool me) {
-	ShowPremium(controller, Ref::Gift::Serialize({ peer->id, months, me }));
+	ShowPremium(controller, Ref::Gift::Serialize({ peer->id, days, me }));
 }
 
 void ShowEmojiStatusPremium(
@@ -1792,6 +1806,8 @@ std::vector<PremiumFeature> PremiumFeaturesOrder(
 			return PremiumFeature::Effects;
 		} else if (s == u"todo"_q) {
 			return PremiumFeature::TodoLists;
+		} else if (s == u"peer_colors"_q) {
+			return PremiumFeature::PeerColors;
 		}
 		return PremiumFeature::kCount;
 	}) | ranges::views::filter([](PremiumFeature type) {
