@@ -81,6 +81,43 @@ void AbstractButton::mouseReleaseEvent(QMouseEvent *e) {
 	}
 }
 
+bool AbstractButton::isSubmitEvent(not_null<QKeyEvent*> e) const {
+	return !e->isAutoRepeat()
+		&& (e->key() == Qt::Key_Space
+			|| e->key() == Qt::Key_Return
+			|| e->key() == Qt::Key_Enter);
+}
+
+void AbstractButton::keyPressEvent(QKeyEvent *e) {
+	if (isSubmitEvent(e)) {
+		setDown(
+			true,
+			StateChangeSource::ByPress,
+			e->modifiers(),
+			Qt::LeftButton);
+		e->accept();
+	} else {
+		RpWidget::keyPressEvent(e);
+	}
+}
+
+void AbstractButton::keyReleaseEvent(QKeyEvent *e) {
+	if (isSubmitEvent(e)) {
+		e->accept();
+		if (isDown()) {
+			setDown(
+				false,
+				StateChangeSource::ByPress,
+				e->modifiers(),
+				Qt::LeftButton);
+
+			clicked(e->modifiers(), Qt::LeftButton);
+		}
+	} else {
+		RpWidget::keyReleaseEvent(e);
+	}
+}
+
 void AbstractButton::clicked(
 		Qt::KeyboardModifiers modifiers,
 		Qt::MouseButton button) {
@@ -130,7 +167,12 @@ bool AbstractButton::setDown(
 		&& (_acceptBoth || button == Qt::LeftButton)) {
 		auto was = _state;
 		_state |= StateFlag::Down;
+
+		const auto weak = base::make_weak(this);
 		onStateChanged(was, source);
+		if (weak) {
+			accessibilityStateChanged({ .pressed = true });
+		}
 		return true;
 	} else if (!down && (_state & StateFlag::Down)) {
 		const auto was = _state;
@@ -139,6 +181,7 @@ bool AbstractButton::setDown(
 		const auto weak = base::make_weak(this);
 		onStateChanged(was, source);
 		if (weak) {
+			accessibilityStateChanged({ .pressed = true });
 			if (was & StateFlag::Over) {
 				clicked(modifiers, button);
 			} else {
@@ -175,4 +218,8 @@ void AbstractButton::clearState() {
 	onStateChanged(was, StateChangeSource::ByUser);
 }
 
+AccessibilityState AbstractButton::accessibilityState() const {
+	return { .pressed = isDown() };
+	}
+
 } // namespace Ui
