@@ -7,11 +7,13 @@
 #include "ui/rp_widget.h"
 
 #include "base/qt_signal_producer.h"
+#include "ui/accessible/ui_accessible_widget.h"
 #include "ui/gl/gl_detection.h"
 
 #include <QtGui/QWindow>
 #include <QtGui/QtEvents>
 #include <QtGui/QColorSpace>
+#include <QtGui/QPainter>
 #include <QtWidgets/QApplication>
 
 namespace Ui {
@@ -127,6 +129,14 @@ rpl::producer<QRect> RpWidgetWrap::paintRequest() const {
 	return eventStreams().paint.events();
 }
 
+void RpWidgetWrap::paintOn(Fn<void(QPainter&)> callback) {
+	const auto widget = rpWidget();
+	paintRequest() | rpl::start_with_next([=] {
+		auto p = QPainter(widget);
+		callback(p);
+	}, lifetime());
+}
+
 rpl::producer<> RpWidgetWrap::alive() const {
 	return eventStreams().alive.events();
 }
@@ -313,6 +323,12 @@ auto RpWidgetWrap::eventStreams() const -> EventStreams& {
 	return *_eventStreams;
 }
 
+void AccessibilityState::writeTo(QAccessible::State &state) {
+	state.checkable = checkable ? 1 : 0;
+	state.checked = checked ? 1 : 0;
+	state.pressed = pressed ? 1 : 0;
+}
+
 RpWidget::RpWidget(QWidget *parent)
 : RpWidgetBase<QWidget>(parent) {
 	[[maybe_unused]] static const auto Once = [] {
@@ -331,4 +347,52 @@ RpWidget::RpWidget(QWidget *parent)
 	}();
 }
 
+QAccessibleInterface *RpWidget::accessibilityCreate() {
+	return (accessibilityRole() != QAccessible::Role::NoRole)
+		? new Accessible::Widget(this)
+		: nullptr;
+}
+
+QAccessible::Role RpWidget::accessibilityRole() {
+	return QAccessible::Role::NoRole;
+}
+
+QString RpWidget::accessibilityName() {
+	return QWidget::accessibleName();
+}
+
+void RpWidget::accessibilityNameChanged() {
+	QAccessibleEvent event(this, QAccessible::NameChanged);
+	QAccessible::updateAccessibility(&event);
+}
+
+QString RpWidget::accessibilityDescription() {
+	return QWidget::accessibleDescription();
+}
+
+void RpWidget::accessibilityDescriptionChanged() {
+	QAccessibleEvent event(this, QAccessible::DescriptionChanged);
+	QAccessible::updateAccessibility(&event);
+}
+
+AccessibilityState RpWidget::accessibilityState() const {
+	return {};
+}
+
+void RpWidget::accessibilityStateChanged(AccessibilityState changes) {
+	auto fields = QAccessible::State();
+	changes.writeTo(fields);
+	QAccessibleStateChangeEvent event(this, fields);
+	QAccessible::updateAccessibility(&event);
+}
+
+QString RpWidget::accessibilityValue() const {
+	return QString();
+}
+
+void RpWidget::accessibilityValueChanged() {
+	QAccessibleValueChangeEvent event(this, accessibilityValue());
+	QAccessible::updateAccessibility(&event);
+}
+
 } // namespace Ui
