@@ -94,6 +94,7 @@ void LinkButton::paintEvent(QPaintEvent *e) {
 void LinkButton::setText(const QString &text) {
 	_text = text;
 	_textWidth = _st.font->width(_text);
+	accessibilityNameChanged();
 	resizeToText();
 	update();
 }
@@ -236,6 +237,7 @@ FlatButton::FlatButton(
 
 void FlatButton::setText(const QString &text) {
 	_text = text;
+	accessibilityNameChanged();
 	update();
 }
 
@@ -303,6 +305,7 @@ RoundButton::RoundButton(
 , _roundRectOver(st.radius ? st.radius : st::buttonRadius, _st.textBgOver) {
 	_textFull.value(
 	) | rpl::start_with_next([=](const TextWithEntities &text) {
+		accessibilityNameChanged();
 		resizeToText(text);
 	}, lifetime());
 }
@@ -360,6 +363,11 @@ void RoundButton::setPenOverride(std::optional<QPen> pen) {
 	update();
 }
 
+void RoundButton::setTextFgOverride(std::optional<QColor> textFg) {
+	_textFgOverride = std::move(textFg);
+	update();
+}
+
 void RoundButton::finishNumbersAnimation() {
 	if (_numbers) {
 		_numbers->finishAnimating();
@@ -506,10 +514,19 @@ void RoundButton::paintEvent(QPaintEvent *e) {
 		: (textTop + _st.iconPosition.y());
 	const auto widthForText = std::max(innerWidth - addedWidth(), 0);
 	if (!_text.isEmpty()) {
-		p.setPen((over || down) ? _st.textFgOver : _st.textFg);
+		if (_textFgOverride) {
+			p.setPen(*_textFgOverride);
+		} else {
+			p.setPen((over || down) ? _st.textFgOver : _st.textFg);
+		}
+		auto local = st::defaultTextPalette;
+		local.linkFg = (over || down)
+			? _st.numbersTextFgOver
+			: _st.numbersTextFg;
 		_text.draw(p, {
 			.position = { textLeft, textTop },
 			.availableWidth = widthForText,
+			.palette = &local,
 			.elisionLines = 1,
 		});
 	}
@@ -963,6 +980,7 @@ void SettingsButton::onStateChanged(
 
 void SettingsButton::setText(TextWithEntities &&text) {
 	_text.setMarkedText(_st.style, text, kMarkupTextOptions, _context);
+	accessibilityNameChanged();
 	update();
 }
 
