@@ -34,6 +34,13 @@ public:
 	void setText(const QString &text);
 	void setColorOverride(std::optional<QColor> textFg);
 
+	QAccessible::Role accessibilityRole() override {
+		return QAccessible::Role::Link;
+	}
+	QString accessibilityName() override {
+		return _text;
+	}
+
 protected:
 	void paintEvent(QPaintEvent *e) override;
 
@@ -102,6 +109,10 @@ class FlatButton : public RippleButton {
 public:
 	FlatButton(QWidget *parent, const QString &text, const style::FlatButton &st);
 
+	QString accessibilityName() override {
+		return _text;
+	}
+
 	void setText(const QString &text);
 	void setWidth(int w);
 	void setColorOverride(std::optional<QColor> color);
@@ -131,6 +142,14 @@ public:
 		rpl::producer<QString> text,
 		const style::RoundButton &st);
 
+	QString accessibilityName() override {
+		return _textFull.current().text;
+	}
+
+	[[nodiscard]] const style::RoundButton &st() const {
+		return _st;
+	}
+
 	void setText(rpl::producer<QString> text);
 	void setText(rpl::producer<TextWithEntities> text);
 	void setContext(const Text::MarkedContext &context);
@@ -144,6 +163,7 @@ public:
 	void setWidthChangedCallback(Fn<void()> callback);
 	void setBrushOverride(std::optional<QBrush> brush);
 	void setPenOverride(std::optional<QPen> pen);
+	void setTextFgOverride(std::optional<QColor> textFg);
 	void finishNumbersAnimation();
 
 	[[nodiscard]] int contentWidth() const;
@@ -181,6 +201,7 @@ private:
 	const style::RoundButton &_st;
 	std::optional<QBrush> _brushOverride;
 	std::optional<QPen> _penOverride;
+	std::optional<QColor> _textFgOverride;
 	RoundRect _roundRect;
 	RoundRect _roundRectOver;
 	Text::MarkedContext _context;
@@ -281,6 +302,10 @@ public:
 		const style::SettingsButton &st = st::defaultSettingsButton);
 	~SettingsButton();
 
+	QString accessibilityName() override {
+		return _text.toString();
+	}
+
 	SettingsButton *toggleOn(
 		rpl::producer<bool> &&toggled,
 		bool ignoreClick = false);
