@@ -87,15 +87,16 @@ void InnerDropdown::resizeToContent() {
 }
 
 void InnerDropdown::resizeEvent(QResizeEvent *e) {
-	_scroll->setGeometry(rect().marginsRemoved(_st.padding).marginsRemoved(_st.scrollMargin));
-	if (auto widget = static_cast<TWidget*>(_scroll->widget())) {
+	_scroll->setGeometry(
+		rect().marginsRemoved(_st.padding).marginsRemoved(_st.scrollMargin));
+	if (auto widget = static_cast<RpWidget*>(_scroll->widget())) {
 		widget->resizeToWidth(_scroll->width());
 		scrolled();
 	}
 }
 
 void InnerDropdown::scrolled() {
-	if (auto widget = static_cast<TWidget*>(_scroll->widget())) {
+	if (auto widget = static_cast<RpWidget*>(_scroll->widget())) {
 		int visibleTop = _scroll->scrollTop();
 		int visibleBottom = visibleTop + _scroll->height();
 		widget->setVisibleTopBottom(visibleTop, visibleBottom);
@@ -362,7 +363,7 @@ bool InnerDropdown::eventFilter(QObject *obj, QEvent *e) {
 
 int InnerDropdown::resizeGetHeight(int newWidth) {
 	auto newHeight = _st.padding.top() + _st.scrollMargin.top() + _st.scrollMargin.bottom() + _st.padding.bottom();
-	if (auto widget = static_cast<TWidget*>(_scroll->widget())) {
+	if (auto widget = static_cast<RpWidget*>(_scroll->widget())) {
 		auto containerWidth = newWidth - _st.padding.left() - _st.padding.right() - _st.scrollMargin.left() - _st.scrollMargin.right();
 		widget->resizeToWidth(containerWidth);
 		newHeight += widget->height();
@@ -373,7 +374,11 @@ int InnerDropdown::resizeGetHeight(int newWidth) {
 	return newHeight;
 }
 
-InnerDropdown::Container::Container(QWidget *parent, object_ptr<TWidget> child, const style::InnerDropdown &st) : TWidget(parent)
+InnerDropdown::Container::Container(
+	QWidget *parent,
+	object_ptr<RpWidget> child,
+	const style::InnerDropdown &st)
+: RpWidget(parent)
 , _child(std::move(child))
 , _st(st) {
 	_child->setParent(this);
@@ -389,7 +394,7 @@ void InnerDropdown::Container::visibleTopBottomUpdated(
 void InnerDropdown::Container::resizeToContent() {
 	auto newWidth = _st.scrollPadding.left() + _st.scrollPadding.right();
 	auto newHeight = _st.scrollPadding.top() + _st.scrollPadding.bottom();
-	if (auto child = static_cast<TWidget*>(children().front())) {
+	if (auto child = static_cast<RpWidget*>(children().front())) {
 		newWidth += child->width();
 		newHeight += child->height();
 	}
