@@ -56,7 +56,7 @@ MenuCallback CreateAddActionCallback(not_null<Ui::PopupMenu*> menu) {
 		} else if (a.isAttention) {
 			auto owned = base::make_unique_q<Ui::Menu::Action>(
 				menu,
-				st::menuWithIconsAttention,
+				a.icon ? st::menuWithIconsAttention : st::menuAttention,
 				Ui::Menu::CreateAction(
 					menu->menu().get(),
 					a.text,
@@ -127,7 +127,7 @@ MenuCallback CreateAddActionCallback(not_null<Ui::DropdownMenu*> menu) {
 		} else if (a.isAttention) {
 			auto owned = base::make_unique_q<Ui::Menu::Action>(
 				menu,
-				st::menuWithIconsAttention,
+				a.icon ? st::menuWithIconsAttention : st::menuAttention,
 				Ui::Menu::CreateAction(
 					menu->menu().get(),
 					a.text,
