@@ -742,9 +742,13 @@ void PopupMenu::showAnimated(PanelAnimation::Origin origin) {
 }
 
 void PopupMenu::hideAnimated() {
-	if (isHidden()) return;
-	if (_hiding) return;
-
+	if (isHidden() || _hiding) {
+		return;
+	}
+	if (!_keepingDelayedActivationPaused) {
+		_keepingDelayedActivationPaused = true;
+		KeepDelayedActivationPaused(true);
+	}
 	startOpacityAnimation(true);
 }
 
@@ -1099,6 +1103,9 @@ PopupMenu::~PopupMenu() {
 			ActivateWindowDelayed(parent);
 		}
 	}
+	if (_keepingDelayedActivationPaused) {
+		KeepDelayedActivationPaused(false);
+	}
 	if (_destroyedCallback) {
 		_destroyedCallback();
 	}
